/* This file is part of the code used to analyze data for 
Pitchers et al. 2018 (or 2019), A multivariate genome-wide association 
study of wing shape in Drosophila melanogaster to be published in Genetics. */

* Uses mega-aligned data to test and choose control adjustements;
* These adjustments are applied in pipeline2 programs. ;
* Two different adjustments are calculated - the linear regression of controls on mifepristone, and
* the total effects of each level of mifepristone.  ;

libname alldata "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data";
libname base "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data";
libname dict "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\Dictionary";

PROC IMPORT OUT= WORK.plmbase00
     DATAFILE=  "D:\Dropbox\DH\Flywing\DGRP\GWAS Programs and data\Dictionary\Dictionary_slid_lmdata.dat"
     DBMS=TAB REPLACE;
     GETNAMES=YES;
 	 GUESSINGROWS=50000;
RUN;
proc sort data=plmbase00;
  	by insert driver direction sex rulevel;
run;

/*
proc freq data=plmbase00(where=(direction ne 'N'));
 	*tables insert*rulevel*sex / nopercent norow nocol nocum;
 	tables insert*rulevel / nopercent norow nocol nocum;
 	*tables insert*tags / nopercent norow nocol nocum;
 	*tables insert*direction*sex / nopercent norow nocol nocum;
 	*tables insert*direction*rulevel / nopercent norow nocol nocum;
 	*tables insert*direction*vial / nopercent norow nocol nocum;
 	*tables insert*direction*sex*rulevel / nopercent norow nocol nocum;
 	*tables fgenotype*mgenotype / nopercent norow nocol nocum;
run;
*/


data plmbase0;
 set plmbase00;
 where (insert not in(/*'YV','W1118',*/'OREW','NGS5','BDM2')) and driver not in('OREW') and (direction ne 'N');
 if insert eq 'STAT92E' and rulevel=2.7 then rulevel=0;
run;

data dictwinf;
  	length insert driver $10. direction $1. rulevel 8. sex $1. date $12. logRUlevel csmm logcs 8. exp $21. treat $36. ;
  	format control $20.;
 	set plmbase0;
  	array coords [*] x1-x48 y1-y48;
	csmm=CS; *scale;
	logcs=log10(10**(log10(CS)));
	do i = 1 to 96;
		coords[i]=100*coords[i];
	end;
  	rep=1;
  	source=substr(insert,1,1);
  	control='Expt';
  	if insert=driver then do;
    	if insert='NGS5' then do;
			control='CNGNG'; *NGS5xNGS5;
			*if year(date)=2012 then rep=2;
		end;
        else if insert='OREW' then do;
			control='C++'; *OREW*OREW;
			*if year(date)=2012 then rep=2;
		end;
		else if insert='YV' then do;
			control='C++'; *yv x yv;
			*if year(date)=2012 then rep=2;
			source='B';
		end;
		else if insert='W1118' then do;
			control='C++'; *w1118 x w1118;
			*if year(date)=2012 then rep=2;
			source='V';
		end;
  		else control='CRiRi'; *RNAixRNAi;
   	end;
  	else do;
		if insert='YV' then do;
			if driver='' then control='C++'; *yv x yv;
	  		if driver='NGS5' then control='CNyv'; *yv x NGS5;
			source='B';
	  	end;
    	else if insert='W1118' then do;
			if driver='' then control='C++'; *w1118 x w1118;
  			if driver='NGS5' then control='CNw1118'; *w1118 x NGS5;
			*if year(date)=2012 then rep=2;
			source='V';
 	 	end;
     	else if insert='OREW' then do;
			if driver='' then control='C++'; *OREWxOREW;
  			if driver='NGS5' then control='CNOREW'; *OREWxNGS5;
  		end;
  	end;
  	tif=substr(file,1,index(file,'.TIF')-1);
  	exp=compress(insert || '-'|| driver);
  	treat=compress(insert || '-'|| driver || '-' || direction || '-' || rep);
  	trsex=compress(insert || '-'|| driver || '-' || direction || '-' || rep || '-' || rulevel || '-' || sex);
  	flyunique=compress(insert || '-'|| driver || '-' || direction || '-' || rep || '-' || rulevel || '-' || sex || '-' || tif); 
  	if rulevel=0 then logRUlevel=-1.4771;
  	else logRUlevel=log10(RUlevel);
 	drop i CS;
run;
proc sort data=dictwinf;
	by treat;
run;

* Computing shape variables (projections onto canonical space);
proc score data=Dictwinf score=base.interlableigdec out=baseDictScores;
  var x1-x48 y1-y48;
run;

proc freq data=baseDictScores;
	tables treat / out=treats;
run;
data treats;
 	set treats;
  	numexp=_N_;
run;
data _NULL_;
	if 0 then set treats nobs=n;
	call symputx('nexps',n);
	stop;
run;
proc sort data=baseDictScores;
 	by insert date direction sex rulevel;
run;

proc means data=baseDictScores noprint;
 by insert date direction sex rulevel;
 id exp treat control cpfile file perp tags vial transfer scale rep source tif trsex flyunique;
 var logRUlevel csmm logcs Prin:;
 output out=medianDictScores median=logRUlevel mcsmm mlogcs mPrin1-mPrin58 ;
run;
proc sort data=medianDictScores;
 by treat exp control source;
run;

* Regression of all experiments and controls on logRU (directions separately);
ods listing close;
ods output ParameterEstimates=regcoeff Estimates=regest;
proc glm data=medianDictScores;
  	class sex;
  	model mPrin1-mPrin58 = logRUlevel /*sex*/ / solution;
	weight _FREQ_;
  	* manova H=logRUlevel;
  	* manova H=sex;
  	by treat exp control source;

	estimate '0'   intercept 1 logRULevel -1.4771;
	estimate '0.1' intercept 1 logRULevel -1.0000/E;
	estimate '0.3' intercept 1 logRULevel -0.5229;
	estimate '0.9' intercept 1 logRULevel -0.0458;
	estimate '2.7' intercept 1 logRULevel  0.4314;
run;
ods output close;
quit;
ods listing;
run;
proc sort data=regcoeff;
 by treat exp control source;
run;
proc transpose data=regcoeff(where=(Parameter='logRUlevel') keep=treat exp control source Dependent Parameter Estimate) out=vects58;
 var Estimate;
 id Dependent;
 by treat exp control source;
run;
proc sort data=regest(drop=StdErr tValue Probt);
 by treat exp control source Parameter;
run;
proc transpose data=regest out=ruvect prefix=mPrin;
 by treat exp control source Parameter;
 var Estimate;
run;
data ruvect;
 set ruvect;
 rename exp=Expt;
 type=substr(control,1,2);
 rulevel=compress('reg'||Parameter);
run;
data vects58;
 set vects58;
 drop _name_;
 rename exp=Expt;
 type=substr(control,1,2);
 rulevel='reg';
run;
data vects58;
 set ruvect vects58;
 drop Parameter _NAME_;
run;

* Derivation of RUlevel LSmeans controls;
ods listing close;
ods output LSMeans=rumeans;
proc glm data=medianDictScores;
  	class RUlevel sex;
  	model mPrin1-mPrin58 = RUlevel /*sex*/;
  	* manova H=logRUlevel;
  	* manova H=sex;
	weight _FREQ_;
  	by treat exp control source;
	lsmeans RUlevel;
run;
ods output close;
quit;
ods listing;
data rumeans;
	set rumeans;
	treats=compress(treat||'-'||rulevel);
	rename mPrin1LSMEAN=mPrin1 mPrin2LSMEAN=mPrin2 mPrin3LSMEAN=mPrin3 mPrin4LSMEAN=mPrin4 mPrin5LSMEAN=mPrin5 mPrin6LSMEAN=mPrin6 mPrin7LSMEAN=mPrin7 mPrin8LSMEAN=mPrin8 
		   mPrin9LSMEAN=mPrin9 mPrin10LSMEAN=mPrin10 mPrin11LSMEAN=mPrin11 mPrin12LSMEAN=mPrin12 mPrin13LSMEAN=mPrin13 mPrin14LSMEAN=mPrin14 mPrin15LSMEAN=mPrin15 
		   mPrin16LSMEAN=mPrin16 mPrin17LSMEAN=mPrin17 mPrin18LSMEAN=mPrin18 mPrin19LSMEAN=mPrin19 mPrin20LSMEAN=mPrin20 mPrin21LSMEAN=mPrin21 mPrin22LSMEAN=mPrin22 
	  	   mPrin23LSMEAN=mPrin23 mPrin24LSMEAN=mPrin24 mPrin25LSMEAN=mPrin25 mPrin26LSMEAN=mPrin26 mPrin27LSMEAN=mPrin27 mPrin28LSMEAN=mPrin28 mPrin29LSMEAN=mPrin29 
		   mPrin30LSMEAN=mPrin30 mPrin31LSMEAN=mPrin31 mPrin32LSMEAN=mPrin32 mPrin33LSMEAN=mPrin33 mPrin34LSMEAN=mPrin34 mPrin35LSMEAN=mPrin35 mPrin36LSMEAN=mPrin36 
		   mPrin37LSMEAN=mPrin37 mPrin38LSMEAN=mPrin38 mPrin39LSMEAN=mPrin39 mPrin40LSMEAN=mPrin40 mPrin41LSMEAN=mPrin41 mPrin42LSMEAN=mPrin42 mPrin43LSMEAN=mPrin43 
		   mPrin44LSMEAN=mPrin44 mPrin45LSMEAN=mPrin45 mPrin46LSMEAN=mPrin46 mPrin47LSMEAN=mPrin47 mPrin48LSMEAN=mPrin48 mPrin49LSMEAN=mPrin49 mPrin50LSMEAN=mPrin50 
	  	   mPrin51LSMEAN=mPrin51 mPrin52LSMEAN=mPrin52 mPrin53LSMEAN=mPrin53 mPrin54LSMEAN=mPrin54 mPrin55LSMEAN=mPrin55 mPrin56LSMEAN=mPrin56 mPrin57LSMEAN=mPrin57 
		   mPrin58LSMEAN=mPrin58;
	drop Effect;
	rename exp=Expt;
	type=substr(control,1,2);
run;
proc means data=rumeans;
	var mPrin:;
	by treat expt control source;
	output out=ruglobal;
run;
data ruglobal;
	set ruglobal(where=(_STAT_='MEAN'));
	drop _TYPE_ _FREQ_ _STAT_;
	treat=compress(treat||'-ave');
	rulevel='ave'; * pooled;
	type=substr(control,1,2);
run;
data rumeans;
	set rumeans(drop=treat rename=(treats=treat)) ruglobal;
	by treat;
run;

* Combining everything;
data vects58;
	set vects58 rumeans;
run;
proc sort data=vects58;
	by treat expt control source;
run;

data vectlab;
	set vects58;
	drop mPrin:;
	i=_N_;
run;
proc transpose data=vects58(keep=mPrin:) out=tvectn;
run;
proc transpose data=tvectn out=vectn;
	var COL:;
	id _NAME_;
run;

* Exports unadjusted RNAi vectors and control vectors;
data dict.vects58;
	merge vectlab(drop=i) vectn(drop=_NAME_);
run;
/* PROC EXPORT
	DATA= dict.vects58
    DBMS=TAB REPLACE
	OUTFILE= "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\Dictionary\vects58.dat";
RUN; */

* Exports projected data (experiments and controls);
data dict.medianDictScores;
 	set medianDictScores;
 	by treat;
run;
/*
PROC EXPORT
	DATA= dict.medianDictScores
    DBMS=TAB REPLACE
	OUTFILE= "D:\Flywing\Dictionary\Results\basedata\medianDictScores.dat";
RUN;
*/

* Estimate lengths of unadjusted vectors;
data length58;
 	set vects58;
	length=sqrt(uss(of mPrin:));
run;
data length58;
	set length58;
	drop mPrin:;
run;

/**** Squared correlations among RNAi and control vectors;  Diagnosing which controls are needed. 
data _NULL_;
	if 0 then set vects58 nobs=n;
	call symputx('nvect',n);
	stop;
run;
proc iml;
	use vects58;
	read all into vects;
	close vects58;
	
	vects=vects`;
 	correffs = j((&nvect*(&nvect-1))/2,3);
 	numcorr=0;
 	do i = 1 to &nvect-1;
   		do j= i+1 to &nvect;
 	  		numcorr=numcorr+1;
	  		correffs[numcorr,1]=i;
	  		correffs[numcorr,2]=j;
 			**** Calculate their correlation.;
			correffs[numcorr,3]=(vects[,i]` * vects[,j]/sqrt(vects[,i]` * vects[,i] # vects[,j]` * vects[,j]))**2;
 		end; 
 	end;
 	create correffs from correffs[colname={i,j,corr}];
  	append from correffs;
 quit;

data corr1;
 	merge correffs vectlab (rename=(treat=treat1 control=control1 type=type1 source=source1 expt=expt1 rulevel=rulevel1));
  	by i;
run;
proc sort;
   	by j;
run;
data corrlabel;
 	merge corr1(in=A) vectlab(in=B rename=(i=j treat=treat2 control=control2 type=type2 source=source2 expt=expt2 rulevel=rulevel2));
  	by j;
  	if A and B;
run;

proc sort data=corrlabel;
  	by treat1;
run;
proc sort data=length58;
  	by treat;
run;
data corrlen1;
  	merge corrlabel length58(rename=(treat=treat1 length=length1 expt=expt1 rulevel=rulevel1) drop=type control source);
  	by treat1;
run;
proc sort data=corrlen1;
  	by treat2;
run;

data dict.basecorrlen;
  	length i j 8. control1 type1 source1 control2 type2 source2 treat1 treat2 $36. length1 length2 corr 8.;
  	merge corrlen1(in=A) length58(in=B rename=(treat=treat2 length=length2 expt=expt2 rulevel=rulevel2) drop=type control source);
  	by treat2;
  	if A & B;
	comp=compress(substr(expt1,1,index(expt1,'-')-1)||'-'||substr(expt2,1,index(expt2,'-')-1));
run;
proc sort data=dict.basecorrlen;
	by descending corr;
run; */
/*
PROC EXPORT
	DATA= dict.basecorrlen
    DBMS=CSV REPLACE
	OUTFILE= "D:\Flywing\Dictionary\Results\basedata\basecorrlen.csv";
RUN;
*/


/* data within;
	set dict.basecorrlen;
 	where (type1='CN' or type2='CN') and (type1='Ex' or type2='Ex');
	rename source1=exBackg source2=cnBackg rulevel1=exRU rulevel2=cnRU;
run;
proc sort data=within;
	by exBackg cnBackg exRU cnRU;
run;
proc univariate plot;
  	var corr;
  	by exBackg cnBackg exRU cnRU;
run;
proc ttest;
  	var corr;
run;
*/
* Mif signal is evident as correlations between controls and experimentals are stronger at the same Mif level and decay as Mif levels become more different;

***** Verdict: Use independent per-RUlevel contrats to estimate control adjustment for treatments;
data controldat;
	set dict.medianDictScores;
	where control ne 'Expt';
run;
proc sort data=controldat;
 	by source sex rulevel;
run;
ods listing close;
ods output ParameterEstimates=ResponseVector Estimates=VectorEstimates;
proc glm data=controldat;
  	class sex direction;
	by source;
  	model mPrin1-mPrin58 = logRULevel sex direction /solution;
	
	estimate '0'   intercept 1 logRULevel -1.4771;
	estimate '0.1' intercept 1 logRULevel -1.0000/E;
	estimate '0.3' intercept 1 logRULevel -0.5229;
	estimate '0.9' intercept 1 logRULevel -0.0458;
	estimate '2.7' intercept 1 logRULevel  0.4314;
run;
ods output close;
quit;
ods listing;
proc sort data=VectorEstimates(drop=StdErr tValue Probt);
 by source Parameter;
run;
proc transpose data=VectorEstimates out=CNvect prefix=CNPrin;
 by source Parameter;
 var Estimate;
run;
data CNvect;
 set CNvect(drop=_NAME_ rename=(Parameter=rulevel));
 if source eq 'B' then Control=compress('CNyv-'||substr(rulevel,1,1)||substr(rulevel,3,1));
 else Control=compress('CNw1118-'||substr(rulevel,1,1)||substr(rulevel,3,1));
run;
* Exports control adjustment vectors to apply to future genes;
data dict.regRUControls;
	merge CNvect(drop=CNPrin:) CNvect(keep=CNPrin:);
run;
PROC EXPORT
	DATA= dict.regRUControls
    DBMS=TAB REPLACE
	OUTFILE= "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\Dictionary\regRUControls.dat";
	*SHEET="MANOVATests";
RUN;


**alt:LSMeans (RU as a class, each level gets its own estimate);

ods listing close;
ods output LSMeans=VectorEstimates;
proc glm data=controldat;
  	class RULevel sex direction;
	by source;
  	model mPrin1-mPrin58 = RULevel sex direction;
  	*manova H=_ALL_;
	lsmeans RULevel;
run;
ods output close;
quit;
ods listing;
data CNvect;
 format source $1. rulevel $3. Control $11.;
 set VectorEstimates(drop=Effect rename=(mPrin1LSMean=CNPrin1 mPrin2LSMean=CNPrin2 mPrin3LSMean=CNPrin3 mPrin4LSMean=CNPrin4 mPrin5LSMean=CNPrin5 mPrin6LSMean=CNPrin6 mPrin7LSMean=CNPrin7 mPrin8LSMean=CNPrin8 
		mPrin9LSMean=CNPrin9 mPrin10LSMean=CNPrin10 mPrin11LSMean=CNPrin11 mPrin12LSMean=CNPrin12 mPrin13LSMean=CNPrin13 mPrin14LSMean=CNPrin14 mPrin15LSMean=CNPrin15 mPrin16LSMean=CNPrin16 
		mPrin17LSMean=CNPrin17 mPrin18LSMean=CNPrin18 mPrin19LSMean=CNPrin19 mPrin20LSMean=CNPrin20 mPrin21LSMean=CNPrin21 mPrin22LSMean=CNPrin22 mPrin23LSMean=CNPrin23 mPrin24LSMean=CNPrin24 
		mPrin25LSMean=CNPrin25 mPrin26LSMean=CNPrin26 mPrin27LSMean=CNPrin27 mPrin28LSMean=CNPrin28 mPrin29LSMean=CNPrin29 mPrin30LSMean=CNPrin30 mPrin31LSMean=CNPrin31 mPrin32LSMean=CNPrin32 
		mPrin33LSMean=CNPrin33 mPrin34LSMean=CNPrin34 mPrin35LSMean=CNPrin35 mPrin36LSMean=CNPrin36 mPrin37LSMean=CNPrin37 mPrin38LSMean=CNPrin38 mPrin39LSMean=CNPrin39 mPrin40LSMean=CNPrin40 
		mPrin41LSMean=CNPrin41 mPrin42LSMean=CNPrin42 mPrin43LSMean=CNPrin43 mPrin44LSMean=CNPrin44 mPrin45LSMean=CNPrin45 mPrin46LSMean=CNPrin46 mPrin47LSMean=CNPrin47 mPrin48LSMean=CNPrin48 
		mPrin49LSMean=CNPrin49 mPrin50LSMean=CNPrin50 mPrin51LSMean=CNPrin51 mPrin52LSMean=CNPrin52 mPrin53LSMean=CNPrin53 mPrin54LSMean=CNPrin54 mPrin55LSMean=CNPrin55 mPrin56LSMean=CNPrin56 
		mPrin57LSMean=CNPrin57 mPrin58LSMean=CNPrin58));
 if source eq 'B' then Control=compress('CNyv-'||substr(rulevel,1,1)||substr(rulevel,3,1));
 else Control=compress('CNw1118-'||substr(rulevel,1,1)||substr(rulevel,3,1));
run;

* Exports control adjustment vectors to apply to future genes;
data dict.RUControls;
	merge CNvect(drop=CNPrin:) CNvect(keep=CNPrin:);
run;
PROC EXPORT
	DATA= dict.RUControls
    DBMS=TAB REPLACE
	OUTFILE= "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\Dictionary\RUControls.dat";
	*SHEET="MANOVATests";
RUN;


