/* This file is part of the code used to analyze data for 
Pitchers et al. 2018 (or 2019), A multivariate genome-wide association 
study of wing shape in Drosophila melanogaster to be published in Genetics. */

/* This file does an analysis of the RNAi knockdowns vectors (dictionary vectors),
 taking into account the effects of mifepristone in control genotypes.  The control effects are 
 calculated by in Dictionary/pipeline_0_ControlsAnalysis.sas, using two different methods. The results here use
 the non-linear total effects calculated by MANOVA. 
* The analyses are of the ROBUST medians by treatment and sex, rather than means;
 The inputs for this program are:
  Dictionary\Dictionary_slid_lmdata.dat - the raw data for the Dictionary experiments;
  Dictionary\regRUcontrols.sas7bdat - controls fit using the regression of shape on log(mifepristone).
  Dictionary\RUcontrols.sas7bdat - controls fit using a MANOVA of shape on mifepristone as a factor.
*/

PROC IMPORT OUT= WORK.plmbase00
	 DATAFILE= "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\Dictionary\Dictionary_slid_lmdata.dat"
     DBMS=TAB REPLACE;
     GETNAMES=YES;
 	 GUESSINGROWS=50000;
RUN;
proc sort data=plmbase00;
  	by insert date driver direction sex rulevel;
run;
data plmbase00;
	set plmbase00;
	todelete=0;
	format alphadate $12.;
	alphadate=date;
run;
/*
proc freq data=plmbase00(where=(direction ne 'N'));
 	*tables insert*rulevel*sex / nopercent norow nocol nocum;
 	*tables insert*rulevel / nopercent norow nocol nocum;
 	*tables insert*tags / nopercent norow nocol nocum;
 	*tables insert*direction*sex / nopercent norow nocol nocum;
 	*tables insert*direction*rulevel / nopercent norow nocol nocum;
 	*tables insert*direction*vial / nopercent norow nocol nocum;
 	*tables insert*direction*sex*rulevel / nopercent norow nocol nocum;
 	*tables fgenotype*mgenotype / nopercent norow nocol nocum;
	tables rulevel*direction*sex / nopercent norow nocol nocum;
run;
*/
data plmbase0;
 set plmbase00(drop=date);
 where (insert not in(/*'YV','W1118',*/'OREW','NGS5','BDM2')) and driver not in('OREW') and (direction ne 'N');
 * Gene-specific edits;
 if insert eq 'BSTAT92E' and rulevel=2.7 then rulevel=0;
 if insert eq 'BDSH' and rulevel=2.7 then todelete=1;
 if insert eq 'VTRH' and rulevel>0.3 then todelete=1;
 if insert eq 'BKAY' and rulevel=2.7 then todelete=1;
 if (insert eq 'BCHN' and direction eq 'M') or (insert eq 'BCHN' and rulevel>0.3) then todelete=1;
 if insert eq 'BFJ' and alphadate eq '1203' then todelete=1;
 if insert eq 'VMER' and alphadate eq '1203' then todelete=1;
 if insert eq 'BMAD' and alphadate eq '1211' then todelete=1;
 date=alphadate;
run;

data plmbase0;
 set plmbase0(where=(todelete=0));
 drop todelete alphadate;
run;

%macro rnaicomplete(ndim,insrtnm,nland,basedir);
options nomlogic nomprint nosymbolgen;

libname dict "&basedir.";
libname base "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data";
libname basedata "&basedir.";

data dictwinf;
  	format insert $10. driver $10. direction $1. rulevel 8.1 sex $1. date $12. logRUlevel 8.5 csmm 8.4 logcs 8.4 exp $21. treat $36. ;
  	format control $20.;
 	set plmbase0;
  	array coords [*] x1-x48 y1-y48;
	csmm=CS; *scale;
	logcs=log10(csmm);
	do i = 1 to 96;
		coords[i]=100*coords[i];
	end;
  	rep=1;
  	source=substr(insert,1,1);
  	control='Expt';
  	if insert=driver then do;
    	if insert='NGS5' then do;
			control='CNGNG'; *NGS5xNGS5;
			*if year(date)=2012 then rep=2;
		end;
        else if insert='OREW' then do;
			control='C++'; *OREW*OREW;
			*if year(date)=2012 then rep=2;
		end;
		else if insert='YV' then do;
			control='C++'; *yv x yv;
			*if year(date)=2012 then rep=2;
			source='B';
		end;
		else if insert='W1118' then do;
			control='C++'; *w1118 x w1118;
			*if year(date)=2012 then rep=2;
			source='V';
		end;
  		else control='CRiRi'; *RNAixRNAi;
   	end;
  	else do;
		if insert='YV' then do;
			if driver='' then control='C++'; *yv x yv;
	  		if driver='NGS5' then control='CNyv'; *yv x NGS5;
			source='B';
	  	end;
    	else if insert='W1118' then do;
			if driver='' then control='C++'; *w1118 x w1118;
  			if driver='NGS5' then control='CNw1118'; *w1118 x NGS5;
			*if year(date)=2012 then rep=2;
			source='V';
 	 	end;
     	else if insert='OREW' then do;
			if driver='' then control='C++'; *OREWxOREW;
  			if driver='NGS5' then control='CNOREW'; *OREWxNGS5;
  		end;
  	end;
  	tif=substr(file,1,index(file,'.TIF')-1);
  	exp=compress(insert || '-'|| driver);
  	treat=compress(insert || '-'|| driver || '-' || direction || '-' || rep);
  	trsex=compress(insert || '-'|| driver || '-' || direction || '-' || rep || '-' || rulevel || '-' || sex);
  	flyunique=compress(insert || '-'|| driver || '-' || direction || '-' || rep || '-' || rulevel || '-' || sex || '-' || tif); 
  	if rulevel=0 then logRUlevel=-1.4771;
  	else logRUlevel=log10(RUlevel);
 	drop i CS;
run;
proc sort data=dictwinf(where=(Control eq 'Expt'));
	by treat;
run;

* Labels for data;
data efflabel;
 	set dictwinf;
	drop x: y: file cpfile perp tags transfer scale csmm logcs control;
run;
data lmdata;
	merge efflabel dictwinf(keep=x: y: csmm logcs);
run;

* Computing shape variables (projections onto canonical space);
proc score data=lmdata score=base.interlableigdec out=plmdata;
  var x1-x48 y1-y48;
run;
proc freq data=plmdata;
	tables treat / out=treats;
run;
data treats;
 	set treats;
  	numexp=_N_;
run;
data _NULL_;
	if 0 then set treats nobs=n;
	call symputx('nexps',n);
	stop;
run;
proc sort data=plmdata;
 	by treat;
run;

*Import control adjustments;
data CNvect; * MANOVA-based;
	set basedata.RUControls;
run;
data regCNvect; * Regression-based;
	set basedata.regRUControls;
run;

* Data adjustment;
data treatdat;
 	set plmdata(rename=(rulevel=ruleveln));
	rulevel=compress(ruleveln);
run;
proc sort data=treatdat;
	by source rulevel;
run;

*use the folowing to adjust by per-level control vectors;

data adjdat;
	merge treatdat(drop=x: y: ruleveln) CNvect(drop=Control);
	by source rulevel;
	adjPrin1=Prin1-CNPrin1;adjPrin2=Prin2-CNPrin2;adjPrin3=Prin3-CNPrin3;adjPrin4=Prin4-CNPrin4;adjPrin5=Prin5-CNPrin5;adjPrin6=Prin6-CNPrin6;
	adjPrin7=Prin7-CNPrin7;adjPrin8=Prin8-CNPrin8;adjPrin9=Prin9-CNPrin9;adjPrin10=Prin10-CNPrin10;adjPrin11=Prin11-CNPrin11;adjPrin12=Prin12-CNPrin12;
	adjPrin13=Prin13-CNPrin13;adjPrin14=Prin14-CNPrin14;adjPrin15=Prin15-CNPrin15;adjPrin16=Prin16-CNPrin16;adjPrin17=Prin17-CNPrin17;adjPrin18=Prin18-CNPrin18;
	adjPrin19=Prin19-CNPrin19;adjPrin20=Prin20-CNPrin20;adjPrin21=Prin21-CNPrin21;adjPrin22=Prin22-CNPrin22;adjPrin23=Prin23-CNPrin23;adjPrin24=Prin24-CNPrin24;
	adjPrin25=Prin25-CNPrin25;adjPrin26=Prin26-CNPrin26;adjPrin27=Prin27-CNPrin27;adjPrin28=Prin28-CNPrin28;adjPrin29=Prin29-CNPrin29;adjPrin30=Prin30-CNPrin30;
	adjPrin31=Prin31-CNPrin31;adjPrin32=Prin32-CNPrin32;adjPrin33=Prin33-CNPrin33;adjPrin34=Prin34-CNPrin34;adjPrin35=Prin35-CNPrin35;adjPrin36=Prin36-CNPrin36;
	adjPrin37=Prin37-CNPrin37;adjPrin38=Prin38-CNPrin38;adjPrin39=Prin39-CNPrin39;adjPrin40=Prin40-CNPrin40;adjPrin41=Prin41-CNPrin41;adjPrin42=Prin42-CNPrin42;
	adjPrin43=Prin43-CNPrin43;adjPrin44=Prin44-CNPrin44;adjPrin45=Prin45-CNPrin45;adjPrin46=Prin46-CNPrin46;adjPrin47=Prin47-CNPrin47;adjPrin48=Prin48-CNPrin48;
	adjPrin49=Prin49-CNPrin49;adjPrin50=Prin50-CNPrin50;adjPrin51=Prin51-CNPrin51;adjPrin52=Prin52-CNPrin52;adjPrin53=Prin53-CNPrin53;adjPrin54=Prin54-CNPrin54;
	adjPrin55=Prin55-CNPrin55;adjPrin56=Prin56-CNPrin56;adjPrin57=Prin57-CNPrin57;adjPrin58=Prin58-CNPrin58;
run;

/*use the following for unadjusted analyses;
data adjdat;
	merge treatdat(drop=x: y: ruleveln) CNvect(drop=Control);
	by source rulevel;
	adjPrin1=Prin1;adjPrin2=Prin2;adjPrin3=Prin3;adjPrin4=Prin4;adjPrin5=Prin5;adjPrin6=Prin6;
	adjPrin7=Prin7;adjPrin8=Prin8;adjPrin9=Prin9;adjPrin10=Prin10;adjPrin11=Prin11;adjPrin12=Prin12;
	adjPrin13=Prin13;adjPrin14=Prin14;adjPrin15=Prin15;adjPrin16=Prin16;adjPrin17=Prin17;adjPrin18=Prin18;
	adjPrin19=Prin19;adjPrin20=Prin20;adjPrin21=Prin21;adjPrin22=Prin22;adjPrin23=Prin23;adjPrin24=Prin24;
	adjPrin25=Prin25;adjPrin26=Prin26;adjPrin27=Prin27;adjPrin28=Prin28;adjPrin29=Prin29;adjPrin30=Prin30;
	adjPrin31=Prin31;adjPrin32=Prin32;adjPrin33=Prin33;adjPrin34=Prin34;adjPrin35=Prin35;adjPrin36=Prin36;
	adjPrin37=Prin37;adjPrin38=Prin38;adjPrin39=Prin39;adjPrin40=Prin40;adjPrin41=Prin41;adjPrin42=Prin42;
	adjPrin43=Prin43;adjPrin44=Prin44;adjPrin45=Prin45;adjPrin46=Prin46;adjPrin47=Prin47;adjPrin48=Prin48;
	adjPrin49=Prin49;adjPrin50=Prin50;adjPrin51=Prin51;adjPrin52=Prin52;adjPrin53=Prin53;adjPrin54=Prin54;
	adjPrin55=Prin55;adjPrin56=Prin56;adjPrin57=Prin57;adjPrin58=Prin58;
run;
 */
*use the following to adjust by unique per-background control vectors;
/* data adjdat;
	merge treatdat(drop=x: y: ruleveln) regCNvect(drop=Control);
	by source rulevel;
	adjPrin1=Prin1-CNPrin1;adjPrin2=Prin2-CNPrin2;adjPrin3=Prin3-CNPrin3;adjPrin4=Prin4-CNPrin4;adjPrin5=Prin5-CNPrin5;adjPrin6=Prin6-CNPrin6;
	adjPrin7=Prin7-CNPrin7;adjPrin8=Prin8-CNPrin8;adjPrin9=Prin9-CNPrin9;adjPrin10=Prin10-CNPrin10;adjPrin11=Prin11-CNPrin11;adjPrin12=Prin12-CNPrin12;
	adjPrin13=Prin13-CNPrin13;adjPrin14=Prin14-CNPrin14;adjPrin15=Prin15-CNPrin15;adjPrin16=Prin16-CNPrin16;adjPrin17=Prin17-CNPrin17;adjPrin18=Prin18-CNPrin18;
	adjPrin19=Prin19-CNPrin19;adjPrin20=Prin20-CNPrin20;adjPrin21=Prin21-CNPrin21;adjPrin22=Prin22-CNPrin22;adjPrin23=Prin23-CNPrin23;adjPrin24=Prin24-CNPrin24;
	adjPrin25=Prin25-CNPrin25;adjPrin26=Prin26-CNPrin26;adjPrin27=Prin27-CNPrin27;adjPrin28=Prin28-CNPrin28;adjPrin29=Prin29-CNPrin29;adjPrin30=Prin30-CNPrin30;
	adjPrin31=Prin31-CNPrin31;adjPrin32=Prin32-CNPrin32;adjPrin33=Prin33-CNPrin33;adjPrin34=Prin34-CNPrin34;adjPrin35=Prin35-CNPrin35;adjPrin36=Prin36-CNPrin36;
	adjPrin37=Prin37-CNPrin37;adjPrin38=Prin38-CNPrin38;adjPrin39=Prin39-CNPrin39;adjPrin40=Prin40-CNPrin40;adjPrin41=Prin41-CNPrin41;adjPrin42=Prin42-CNPrin42;
	adjPrin43=Prin43-CNPrin43;adjPrin44=Prin44-CNPrin44;adjPrin45=Prin45-CNPrin45;adjPrin46=Prin46-CNPrin46;adjPrin47=Prin47-CNPrin47;adjPrin48=Prin48-CNPrin48;
	adjPrin49=Prin49-CNPrin49;adjPrin50=Prin50-CNPrin50;adjPrin51=Prin51-CNPrin51;adjPrin52=Prin52-CNPrin52;adjPrin53=Prin53-CNPrin53;adjPrin54=Prin54-CNPrin54;
	adjPrin55=Prin55-CNPrin55;adjPrin56=Prin56-CNPrin56;adjPrin57=Prin57-CNPrin57;adjPrin58=Prin58-CNPrin58;
run;
*/
data dict.slid_adjdata_&insrtnm._lm&nland._dim&ndim.;
	set adjdat(where=(adjPrin1^=.));
	drop Prin: CNPrin:;
run;
/* PROC EXPORT
	DATA= dict.slid_adjdata_&insrtnm._lm&nland._dim&ndim.
    DBMS=TAB REPLACE
	OUTFILE= "&basedir.\slid_adjdata_&insrtnm._lm&nland._dim&ndim..dat";
RUN;  */

*** ANALYSIS;
data plmtoan;
 set dict.slid_adjdata_&insrtnm._lm&nland._dim&ndim.(keep= LOGCS INSERT DRIVER DIRECTION RULEVEL LOGRULEVEL VIAL SEX DATE adjPrin:);
run;

* Computation of medians for robust analysis;
proc sort data=plmtoan;
	by insert date direction sex rulevel;
run;

proc means data=plmtoan noprint;
 by insert date direction sex rulevel;
 var logRUlevel logcs adjPrin:;
 output out=plmmedians median=logRUlevel mlogcs madjPrin1-madjPrin58 ;
run;


* MANOVA to compute LSMeans per mif level;
proc sort data=plmmedians;
	by insert date direction sex;
run;

ods listing close;
ods output LSMeans=ModelMeans; *MultStat=MANOVAtests;
proc glm data=plmmedians;
 class direction rulevel sex; *vial;
 by insert date;
 model madjPrin: = /*logcs*/ sex rulevel direction sex*rulevel direction*sex direction*rulevel; *vial;
 *random vial;
 weight _FREQ_;
 lsmeans rulevel sex*rulevel direction*rulevel;
 *manova h= _ALL_;
run;
ods output close;
quit;
ods listing;

data dxsmedians;
 set plmmedians(drop=_type_ _freq_ mlogcs);
 Effect='directio_rulevel_sex';
 rename madjPrin1=madjPrin1LSMean madjPrin2=madjPrin2LSMean madjPrin3=madjPrin3LSMean madjPrin4=madjPrin4LSMean madjPrin5=madjPrin5LSMean madjPrin6=madjPrin6LSMean 
	    madjPrin7=madjPrin7LSMean madjPrin8=madjPrin8LSMean madjPrin9=madjPrin9LSMean madjPrin10=madjPrin10LSMean madjPrin11=madjPrin11LSMean madjPrin12=madjPrin12LSMean 
		madjPrin13=madjPrin13LSMean madjPrin14=madjPrin14LSMean madjPrin15=madjPrin15LSMean madjPrin16=madjPrin16LSMean madjPrin17=madjPrin17LSMean madjPrin18=madjPrin18LSMean 
		madjPrin19=madjPrin19LSMean madjPrin20=madjPrin20LSMean madjPrin21=madjPrin21LSMean madjPrin22=madjPrin22LSMean madjPrin23=madjPrin23LSMean madjPrin24=madjPrin24LSMean 
		madjPrin25=madjPrin25LSMean madjPrin26=madjPrin26LSMean madjPrin27=madjPrin27LSMean madjPrin28=madjPrin28LSMean madjPrin29=madjPrin29LSMean madjPrin30=madjPrin30LSMean 
		madjPrin31=madjPrin31LSMean madjPrin32=madjPrin32LSMean madjPrin33=madjPrin33LSMean madjPrin34=madjPrin34LSMean madjPrin35=madjPrin35LSMean madjPrin36=madjPrin36LSMean 
		madjPrin37=madjPrin37LSMean madjPrin38=madjPrin38LSMean madjPrin39=madjPrin39LSMean madjPrin40=madjPrin40LSMean madjPrin41=madjPrin41LSMean madjPrin42=madjPrin42LSMean 
		madjPrin43=madjPrin43LSMean madjPrin44=madjPrin44LSMean madjPrin45=madjPrin45LSMean madjPrin46=madjPrin46LSMean madjPrin47=madjPrin47LSMean madjPrin48=madjPrin48LSMean 
		madjPrin49=madjPrin49LSMean madjPrin50=madjPrin50LSMean madjPrin51=madjPrin51LSMean madjPrin52=madjPrin52LSMean madjPrin53=madjPrin53LSMean madjPrin54=madjPrin54LSMean 
		madjPrin55=madjPrin55LSMean madjPrin56=madjPrin56LSMean madjPrin57=madjPrin57LSMean madjPrin58=madjPrin58LSMean;
run;
data ModelMeans;
 set dxsmedians ModelMeans;
run;



data ModelMeans;
	merge ModelMeans(drop=madjPrin:) ModelMeans(keep=madjPrin:);
	if direction eq '' then direction='I';
	if sex eq '' then sex='I';
run;
proc sort data=ModelMeans;
 	by INSERT DATE descending EFFECT DIRECTION SEX RULEVEL;
run;


* (Median-based) Regressions to estimate linear response of wings to mif;

*averaging directions and sexes;
proc sort data=plmmedians;
	by insert date direction sex;
run;

ods listing close;
ods output ParameterEstimates=ResponseVector;
proc glm data=plmmedians;
 	class direction sex;
 	by insert date;
 	model madjPrin: = /*logcs*/ logRULevel sex direction /solution;
	weight _FREQ_;
 	*manova h= _ALL_;
 	output out=Resid r=r1-r58;
run;
ods output close;
quit;
ods listing;

proc transpose data=ResponseVector(where=(Parameter='logRUlevel') keep=insert date Dependent Parameter Estimate) out=RUregress;
	var Estimate;
	id Dependent;
	by insert date;
run;
data RUregress;
	set RUregress(drop=madjPrin:);
	drop _NAME_;
	direction = 'I';
	sex = 'I';
	merge RUregress(keep=madjPrin:);
run;


* Updates pooled vector file;
data basedata.pooledRegEstimates;
	set /*basedata.pooledRegEstimates*/ RUregress;
run;
PROC EXPORT
	DATA= basedata.pooledRegEstimates
    DBMS=TAB REPLACE
	OUTFILE= "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\Dictionary\pooledRegEstimates.dat";
RUN;


%mend;

* %macro rnaicomplete(ndim,insrtnm,nland,basedir);
/*%rnaicomplete(58,Bvg,12,D:\Flywing\Dictionary\Results\basedata\data_Bvg_11.11);*/
%rnaicomplete(58,multign,12,D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\Dictionary);

