/* This file is part of the code used to analyze data for 
Pitchers et al. 2018 (or 2019), A multivariate genome-wide association 
study of wing shape in Drosophila melanogaster to be published in Genetics. */

/* This code provides data sets to GLMnet for separate regressions of each focal, signficant SNP. 
  These analyses are then done in R.
* Input data is ;
* alldata.interlabsizejun phenotype data.  - this file is generated by the supplied program MANOVAHIGHQchunk.sas.
		Need to get the subset of lines that are NOT in the inversion under consideration to use here.; 
* alldata.GOf2hq3_05 List of significant post-inversion SNPs, plus closest gene information;
* alldata.f2highq184invmsked  the snp data, post inversion.   - not included. Run SNPdata/ReadSNPquality20.sas;
* alldata.F2ldhq184invmsked SNP data with a MAC>=3 inversion masked;    - not included. Run SNPdata/ReadSNPquality20.sas; 
* alldata.f2ldhq184 SNP data with all data in, including MAC>3 and inversions;   - not included.  Run SNPdata/ReadSNPquality20.sas; 
* alldata.ldpi184 correlations above 0.5 for pairs with one significant snp at 5% FDR. - not included. Run LinkageDiseq/CHCorrHQ184sig.sas;
* GLMnet\DGRPallmac5evec.txt - Scores of the DGRP SNP data on the first 20 PCS computed by smartpca. Used to compensate for population structure.


/* This  incorporates the smartpca scores into the analysis.
This version does not use the hetorzygote data, to keep it as comparable as possible to the previous analyses.
This program analyses the high quality FDR 5% significant SNPs for their correlations
with other sites in the genome.  combined data sets from the Houle and Dworkin labs.  
This then is furnished to GLMnet for analysis using lasso. */

************************************************;
* Change when moving windows-UNIX;
*libname freeze2 '$HOME/DGRP/data.freeze2';  * Unix form;
*libname freeze2  "D:\Dropbox\DH\Flywing\DGRP\data.freeze2";  * Windows form ;
libname alldata "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data";
libname temphqd "D:\DGRPbig\HQrevise"; * This temporary data set can be used to put intermediate results on a fast drive. ;
************************************************;

filename junk dummy;
options mautosource;
*** Windows ;
*** UNIX version ;
*libname freeze2 '$HOME/DGRP/data.freeze2';
*options sasautos=('$HOME/DGRP/SASMacros');
option nolabel;


 ods html close;	
 * ods results off;

 data overall;
 input i 1 ins 3-4;
 datalines;
0 -1
 ;
run;
%macro sysver;
%if &SYSVER=9.3 | &SYSVER=9.4 %then %do;
  proc transpose data=linelsmean out=linelsmean suffix=LSMean;
    var lsmean;
	by linenum;
	id dependent;
run;
%end;
%mend;

%macro datasampsz(dset,returnCode=);
data _null_;
  set &dset nobs=nobs;
 call symput('dn',trim(left(nobs)));
run;
%let returnCode=&dn;
%mend; *datasampsz;

* trylock tests to see if a lock is available on the member named in the call.;
 %macro trylock(member,timeout=1,retry=1);
	 %local starttime;
	 %let starttime = %sysfunc(datetime());
	 %do %until(&syslckrc <= 0
	 or %sysevalf(%sysfunc(datetime()) > (&starttime + &timeout)));
	 %put trying open ...;
	 data _null_;
	 dsid = 0;
	 do until (dsid > 0 or datetime() > (&starttime + &timeout));
	 dsid = open("&member");
	 if (dsid = 0) then rc = sleep(&retry);
	 end;
	 if (dsid > 0) then rc = close(dsid);
	 run;
	 %put trying lock ...;
	 lock &member;
	 %put syslckrc=&syslckrc;
	 %end;
 %mend trylock; 

%macro lsmeans;
%put Calculating Least Squares means for wing data.;

* phenotype data.;
ods output LSMeans=linelsmean Means=linemeans;
proc glm data=alldata.interlabsizejun;  
 class linenum lab sex;
 model Prin1-Prin58 csmm = lab sex linenum / solution;
 means linenum;
 lsmeans linenum;  
 quit;
run;
ods output close;
proc sort data=linelsmean;
 by linenum;
 run;
%sysver;
proc sort data=linemeans;
  by linenum;
run;
data lsmeanN;
 format lnnum 5.;
 merge linelsmean linemeans(keep =linenum N);
 by linenum;
 lnnum=linenum+0;
 if lnnum=378 | lnnum=387 | lnnum=398 | lnnum=476 | lnnum=514 | lnnum=556 | lnnum=591 then delete;
 * We phenotyped these lines, but they are not included in Freeze2 of the DGRP.;
 *drop linenum;
run;
proc sort data= lsmeanN;;
 by lnnum;
 run;
data &libr..lsmeanN;
* data lsmeanN2;
  format DGRP 5.;
   set lsmeanN;
   DGRP=_N_;
   run;
%mend;

%macro snpswinv;
%put Getting the full SNP data.;
data snpswinv;
array f2withdata[184] L21 L26 L28 L38 L40 L41 L42 L45 L57 L59 L69 L73 L75 L83 L85 L88 L91 L93 L101 L105 L109 L129 L136 L138 L142 L149 L153 
L158 L161 L176 L177 L181 L189 L195 L208 L217 L223 L227 L228 L229 L233 L235 L237 L239 L256 L280 L287 L301 L303 L304 L306 L307 L309 L310 L313 L315 
L317 L318 L319 L320 L321 L324 L325 L332 L335 L336 L338 L340 L350 L352 L356 L357 L358 L359 L360 L361 L362 L365 L367 L370 L371 L373 L374 L375 L377 
L379 L380 L381 L382 L383 L385 L386 L391 L392 L399 L405 L406 L409 L426 L427 L437 L439 L440 L441 L443 L461 L486 L491 L492 L502 L508 L509 L513 
L517 L530 L531 L535 L555 L563 L584 L589 L595 L639 L642 L646 L703 L705 L707 L712 L714 L716 L721 L730 L732 L737 L738 L748 L757 L761 L765 L774 L776 
L783 L786 L787 L790 L796 L799 L801 L802 L804 L805 L808 L810 L812 L818 L819 L820 L821 L822 L832 L837 L843 L849 L850 L852 L853 L855 L857 L859 L861 L879 L882 L884 
L887 L890 L892 L894 L897 L900 L907 L908 L911 L913;
array lcons[184] Ln1-Ln184;
set alldata.F2ldhq184;
   do i = 1 to 184;
	 f2withdata[i]=lcons[i];
	end;
 rename nummiss184=nummiss nlines184=nlines mac184=mac psnp184=psnp;
drop Ln1-Ln184 i;
run;
* Adding the inversions and principal component scores .;
PROC IMPORT OUT= WORK.EigsnpPC 
            DATAFILE= "D:\Dropbox\DH\Flywing\DGRP\HQrevise\data\DGRPallmac5evec.txt" 
            DBMS=DLM REPLACE;
     DELIMITER='20'x; 
     GETNAMES=YES;
     DATAROW=2; 
RUN;
data structure;
 set eigsnppc;
  DGRP=_N_;
  linenum=substr(Line,2);
  lnnum=linenum+0;
   drop var22 SC14-SC20;
   rename Line=DGRPline;
run;
/* proc transpose data=EigsnpPC out=structuresc;
 var Sc1-SC13;
 id line;
 run;
data struct;
 length snp 8. stref stalt $10.;
 set structuresc;
 snp=_N_;
	stref=compress('PC'||_NAME_);
	nummiss=0;
	nlines=184;
select (stref);
  when ('PCSc1') stalt='In2Lt';
  when ('PCSc2') stalt='In3RMo';
  when ('PCSC5') stalt='In2RNS';
  otherwise;
end;
run; */
data big3inv;
 set snpswinv;
 if _N_>3 then stop;
run;
%mend;


%macro prepdata;
%put Preparing the basic files for multiple regression.;
* called by regsnp;

* sigfamgenNIMP has the genotypes of all the snps that could be used.  
* sigfamgenIMP has the imputed genotypes of all the snps that could be used.  
* corrgenePI has the list of all the significant SNPs that each significant SNP is correlated to.;
* ingenePI has the signficant snps and the genes they are closest to.;

* GO information.;
data ingenePI;
 set alldata.GOf2hq3_05;
 * Data with the closest 4 genes and the P values.;
 dist=0;
 sigsnp=snp;
 csnp=snp;
 closest=gene1;
 dist2closest=dist1;
 siteclass=class1;
 drop locustype;
* keep closest dist2closest snp siteClass SNPPvalue isWing sigsnp csnp dist;
run;

data focsigsnp;
 set ingenePI;
 focalsig = 1;
 focalsigsnp=snp;
 drop dist csnp closest dist2closest siteclass;
run;
proc sort data=focsigsnp;
 by snp;
 run;

 * Getting relevant LD correlations with a significant SNP.;
 data correlated;
   merge alldata.LDpi184(where=(snp1>3 & snp2>3)) alldata.GOf2hq3_05(in=A rename=(snp=snp1 ininversion=ininv1)); *** This was wrong in the last run.  Should keep only those correlated with
   * significant SNPs.  I THINK this is right now.  The old way would have made bigger files, but not messed up the calculations?;  
   by snp1;
   if A;
	snp=snp1;
	dist = abs(snp1-snp2);
	if dist>90000000 then difchr=1;
	else difchr=0;
	if dist>100000 then far=1;
	else far=0;
	drop arb1 arb2;
run;
* List of significant SNPs that are correlated with something.;
proc sort data=correlated nodupkey out=sigfam;
 by snp;
run;
* List of SNPs correlated with a significant SNP;
data corrwsig;
  set correlated;
  snp=snp2;
  focalsigsnp=snp1;
  keep snp focalsigsnp;
run;
data sigandcorr;
 set focsigsnp corrwsig; * focsigsnp has to be listed first to make things come out right. ;
 if focalsig=. then focalsig=0;
  keep snp focalsigsnp focalsig;
run;
proc sort data=sigandcorr nodupkey out=corrsomesig;
 by snp;
run;

* Now add the actual genotype information.;
data sigfamgenNIMP;
 length snp nummiss nlines mac psnp 8.;
 merge corrsomesig(in=A) snpswinv;
  by snp;
  if A;
run;

* Now impute the actual genotype information for these SNPs.;
data sigfamgenIMP;
array f2withdata[*] L21 L26 L28 L38 L40 L41 L42 L45 L57 L59 L69 L73 L75 L83 L85 L88 L91 L93 L101 L105 L109 
L129 L136 L138 L142 L149 L153 L158 L161 L176 L177 L181 L189 L195 L208 L217 L223 L227 L228 L229 L233 L235 L237 L239 L256 L280 
L287 L301 L303 L304 L306 L307 L309 L310 L313 L315 L317 L318 L319 L320 L321 L324 L325 L332 L335 L336 L338 L340 L350 L352 L356 L357 L358 L359 
L360 L361 L362 L365 L367 L370 L371 L373 L374 L375 L377 L379 L380 L381 L382 L383 L385 L386 L391 L392 L399 L405 L406 L409 L426 L427 L437 
L439 L440 L441 L443 L461 L486 L491 L492 L502 L508 L509 L513 L517 L530 L531 L535 L555 L563 L584 L589 L595 L639 L642 L646 L703 L705 
L707 L712 L714 L716 L721 L730 L732 L737 L738 L748 L757 L761 L765 L774 L776 L783 L786 L787 L790 L796 L799 L801 L802 L804 L805 L808 L810 L812 L818 
L819 L820 L821 L822 L832 L837 L843 L849 L850 L852 L853 L855 L857 L859 L861 L879 L882 L884 L887 L890 L892 L894 L897 L900 L907 L908 L911 L913;
merge snpswinv corrsomesig(in=A);
 by snp;
 if A;
* if focalsig=. then focalsig=0;
* Now impute the missing values as the common allele.;
    do i = 1 to dim(f2withdata);
       if f2withdata[i]=. then f2withdata[i]=psnp;
      * if f2withdata[i]=. then f2withdata[i]=0;
	end;
run;

* Now connect the snps that are significant with their correlated counterparts.;
data corrsnp;
 set correlated;
 snp=snp2;
 drop Pvalue Qvalue chrom snp1 snp2;
run;
proc sort data=corrsnp;
 by snp;
run;
data corrgenePI;
 merge corrsnp(IN=A) focsigsnp(keep=snp focalsig);
 by snp;
 if A;
 if focalsig=. then focalsig=0;
 rename focalsig=secondsig;
 run;
* This expands the list of genes that are analyzed to include multiple genes for each SNP, if it is within 2kb
  of more than one gene.;
data siggenesnp;
 set focsigsnp;
 closest=gene1;
 gcl=1;
 output;
 if (dist2^=. & abs(dist2)<2000) then do;
 closest=gene2;
 gcl=2;
 output;
  end;
  if (dist3^=. & abs(dist3)<2000) then do;
   closest=gene3;
 gcl=3;
   output;
  end;
   if (dist4^=. & abs(dist4)<2000) then do;
     closest=gene4;
 gcl=4;
      output;
  end;
  csnp=sigsnp;
 keep snp focalsig sigsnp closest gcl;
run;

* Make a data set with the list of all the genes with a significant SNP closest or correlated to them. ;
proc freq data=siggenesnp;
 tables closest / out=alldata.geneslist;
run;
%mend; * prepdata;

%macro glmSNP(maxprn,PCs,cutr,enterp,delprev,output,list,libr,finalfile,qfirst,qlast);
dm 'odsresults; clear'; 
**** Prelminaries;
* Get all the input data together appropriately.;
%let numvaranl=%eval(&maxprn+1);
%global dn;
%let nowtime=%sysfunc(time(),time.);
%put output = &output;
%if &output^=out %then %do;
%put Shutting down output.;
			  proc printto log=junk;
			run;
			%let list=no;
ods listing close;
%end;
%else %do;
%put Allowing output.;
ods listing;
proc printto log=log;
	run;
%end;
%datasampsz(alldata.GOf2hq3_05);
%let nsnps=&dn;
 options nomlogic nomprint nosymbolgen;
 %if &delprev=delete %then %do;
	 %if %sysfunc(exist(&libr..snpcorrdat&qfirst.to&qlast)) %then %do;
		proc datasets lib=&libr;
		  delete snpcorrdat&qfirst.to&qlast snpglminf&qfirst.to&qlast;
		  run;
		  quit;
		%end; 
	%end;

		%if ^%sysfunc(exist(&libr..lsmeann)) %then %lsmeans;
		%if ^%sysfunc(exist(work.snpswinv)) %then  %snpswinv;
		%if ^%sysfunc(exist(work.ingenepi))  %then %prepdata;
		**** End Prelminaries;


%if &qlast=all %then %let qlast=&nsnps;
%do q=&qfirst %to &qlast; 
	data wfile;
	 set alldata.GOf2hq3_05(firstobs=&q obs=&q);	
	 targetsig=1;
     focalnum=symget('q')+0;
	 call symput('SNP',snp);
	 keep focalnum snp gene1 cluster clustersize clustrange;
	run; 

**** assemble a list of all the genes relevant to each snp.;
* Get the genes implicated by this SNP.;
 data geneswsnp;
   merge wfile(in=A) siggenesnp;
   by snp;
   if A;
 run;
 %datasampsz(geneswsnp);
 %let ngenes=&dn;
 %let Genelab=;
 			%if %sysfunc(exist(work.allingene)) %then %do;
			proc datasets lib=work;
			  delete allingene2;
			  run;
			  quit;
			%end;


 %do g=1 %to &ngenes;
	data gfile;
	 set geneswsnp(firstobs=&g obs=&g);	
	 genelab=symget('Genelab');
	 closelab = Genelab || closest;
	 call symput('Gene',compress(closest));
	 call symput('Genelab',compress(closelab," -:()"));
	run; 
	%put snp is &q &SNP, Gene &g &Gene. &Genelab;
	data focingene;
		 set ingenePI;
		  where gene1="&Gene" |  (gene2="&Gene" & abs(dist2)<2000) | (gene3="&Gene" & abs(dist2)<2000) | (gene4="&Gene" & abs(dist2)<2000);
		  targsnp=symget('SNP');
		  if snp=targsnp then targetsig=1;  
			else targetsig=0;
		  call symput('Inversion',compress(Inversion));
		 run;
		* Get the correlated SNPs relevant to this gene;
	data corrwgene;
	  set corrgenePI;
	  where gene1="&Gene" | (gene2="&Gene" & abs(dist2)<2000) | (gene3="&Gene" & abs(dist2)<2000) | (gene4="&Gene" & abs(dist2)<2000);
	 run;
	* Put them together in one file.;
	data allinge;
	  set focingene corrwgene;
	  if dist=0 then focalsig=1;
	  else focalsig=0;
	  if targetsig=. then targetsig=0;
	  keep snp focalsig targetsig Pvalue;
	  run;
	proc append base=allingene2 data=allinge;
	run;
%end; *g loop over genes relevant to this SNP.;
proc sort data=allingene2 out=allingene nodupkey;
 by snp;
run;
* Put together with non-imputed genotype data.;
data allingNIMP;
 	merge allingene(IN=a where=(targetsig=1)) sigfamgenNIMP;
	by snp;
	if A;
    *focalgene=symget('Genelab');
    *focinversion=symget('Inversion');
	run;
  * Now put together with the imputed genotype data.;
data allingIMP;
 	merge allingene(IN=a where=(targetsig=0)) sigfamgenIMP;
	by snp;
	if A;
	drop i;
	run;

data allsnp; 
	set allingNIMP allingIMP;
    focinversion=symget('Inversion');
    focalgene=symget('Genelab');
	 snpid=_N_;
  run; 
  %datasampsz(allsnp);
  %let nallsnps=&dn;
  proc transpose data=allsnp out=tallsnp prefix=snp;
   var L21 L26 L28 L38 L40 L41 L42 L45 L57 L59 L69 L73 L75 L83 L85 L88 L91 L93 L101 L105 L109 
	L129 L136 L138 L142 L149 L153 L158 L161 L176 L177 L181 L189 L195 L208 L217 L223 L227 L228 L229 L233 L235 L237 L239 L256 L280 
	L287 L301 L303 L304 L306 L307 L309 L310 L313 L315 L317 L318 L319 L320 L321 L324 L325 L332 L335 L336 L338 L340 L350 L352 L356 L357 L358 L359 
	L360 L361 L362 L365 L367 L370 L371 L373 L374 L375 L377 L379 L380 L381 L382 L383 L385 L386 L391 L392 L399 L405 L406 L409 L426 L427 L437 
	L439 L440 L441 L443 L461 L486 L491 L492 L502 L508 L509 L513 L517 L530 L531 L535 L555 L563 L584 L589 L595 L639 L642 L646 L703 L705 
	L707 L712 L714 L716 L721 L730 L732 L737 L738 L748 L757 L761 L765 L774 L776 L783 L786 L787 L790 L796 L799 L801 L802 L804 L805 L808 L810 L812 L818 
	L819 L820 L821 L822 L832 L837 L843 L849 L850 L852 L853 L855 L857 L859 L861 L879 L882 L884 L887 L890 L892 L894 L897 L900 L907 L908 L911 L913;
  run;
  *options mlogic mprint symbolgen;
  data readydata;
    set tallsnp;
    if snp1=. then do;
	  %if &nallsnps>1 %then %do s=2 %to &nallsnps;
		  snp&s=.;
		%end;
	end; 
      Ln = _N_;
    run;
  proc transpose data=readydata out=tready prefix=Ln;
    var snp1-snp&nallsnps;
  run;
  options nomlogic nomprint nosymbolgen;
  data checklineid;
   set readydata;
   keep DGRP _NAME_;
   DGRP = _N_;
   rename _NAME_=DGRPline;
run;
*Output common data for all SNPs to R and GLMnet.;
data doonce;
   length DGRP lnnum 8. Linenum $3. DGRPline $8.; 
   merge &libr..lsmeann checklineid structure;
   by DGRP;
run;
PROC EXPORT DATA= WORK.DOONCE 
            OUTFILE= "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data\DGRPphenotypes.csv" 
            DBMS=CSV REPLACE;
     PUTNAMES=YES;
RUN;
data missingsnp;
 length DGRP 8.;
  array snps[*] snp1-snp5000;
  set checklineid;
  do i = 1 to 5000;
   snps[i]=.;
  end;
  drop i;
  run;

data allsnpinfo;
 set allsnp;  **** There is now only allsnp, not a nimp and an imp;
 keep snpid snp nlines nummiss mac psnp focalsig targetsig focinversion typep;
run;
data allsnpcorr;
 length focalnum 8.;
  merge allsnpinfo tready;
  focalnum=symget('q')+0;
  drop _NAME_;
run;
  %put nsnps is &nsnps,  nallsnps is &nallsnps;
   data checkmac;
    set allsnpcorr;
     order=_N_;
   run;
%datasampsz(checkmac);
 %let ncsnps=&dn;
data snpinfo;
 length focalnum 8. focinversion $15.;
 set checkmac;
 rename order=snporder;
 keep focalnum order snpid snp nummiss mac psnp focalsig targetsig focinversion typep;
run;

proc transpose data=checkmac out=genesnp prefix=snp;
	  var Ln1-Ln184;
	run;
data genesnp;
 set genesnp;
 DGRP=_N_;
 drop _NAME_;
run;
data linesnp;
 format focalnum numcomp DGRP 5. DGRPline $8.;
 merge missingsnp genesnp;
 by DGRP;
 focalnum=symget('q')+0;
 numcomp=symget('dn')+0;
 if snp1=. then delete;
 run;
 data linesnpwid;
  merge wfile(keep=snp focalnum) linesnp;
   by focalnum;
  run;
proc sort;
  by DGRP;
  run;
		 %if %sysfunc(exist(&libr..snpcorrdat&qfirst.to&qlast)) %then %do;
		     %trylock(&libr..snpcorrdat&qfirst.to&qlast);
		     %trylock(&libr..snpglminf&qfirst.to&qlast);
		 	 proc datasets;
			  append base= &libr..snpcorrdat&qfirst.to&qlast data=linesnpwid force;
			  append base= &libr..snpglminf&qfirst.to&qlast data=wfile force;
			  run;
			  quit; 
		 %end;
		 %else %do;
		   data &libr..snpcorrdat&qfirst.to&qlast;
		     set linesnpwid;
		   run;
		   data &libr..snpglminf&qfirst.to&qlast;
		      set wfile;
		   run;
		  %end;
 dm 'odsresults; clear';  
%if &output^=out %then %do; 
  dm 'clear log';
  dm 'clear output';
%end;
 
%end;  * q; 


  ods html;
  ods results on;
  ods listing; 
  title1; title2;
   proc printto;
	run;
* Move data sets over to the permanent destination, and erase the temporary permanent files.;
 	options spool;
 	proc datasets library=&libr;
	  copy out=alldata move;
	  select snpcorrdat&qfirst.to&qlast snpglminf&qfirst.to&qlast;
	  run;
	  quit;
	  *Proc Export is limited to lrecl<32762;

data ouptup;
file "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data\compsnps&qfirst.to&qlast..csv" dlm='2C'x lrecl=50000;
 set alldata.snpcorrdat297to297;
 if _N_=1 then do;
  put "snp,focalnum,numcomp,DGRP,DGRPline" @;
  %do i = 1 %to 5000;
    put ",snp&i" @;
  %end;
  put;
 end;
 put snp focalnum numcomp DGRP DGRPline $ snp1-snp5000;
 run;
 options nomlogic nomprint nosymbolgen;

/* PROC EXPORT DATA= alldata.snpcorrdat&qfirst.to&qlast 
            OUTFILE= "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data\compsnps&qfirst.to&qlast..csv" 
            DBMS=CSV REPLACE;
     PUTNAMES=YES; */
RUN;
%mend;

* glmSNP((maxprn,PCs,cutr,enterp,delprev,output,list,libr,finalfile,qfirst,qlast);
%glmSNP(58,13,0.99,0.001,delete,out,list,temphqd,GLMnet,297,297);

/* data ouptup;
 file "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data\compsnpstest.csv" dlm='2C'x lrecl=50000;
 if _N_=1 then put "snp,focalnum,numcomp,DGRP,DGRPline";
 set alldata.snpcorrdat297to297;
 * file "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data\compsnps&qfirst.to&qlast..csv";
 put snp focalnum numcomp DGRP DGRPline $ snp1-snp50;
 run; */
