/* This file is part of the code used to analyze data for 
Pitchers et al. 2018 (or 2019), A multivariate genome-wide association 
study of wing shape in Drosophila melanogaster to be published in Genetics. */

* This program calculates the gametic correlations between a set of significant SNPs
(for example in data sets alldata.gof2hq3_05 or glmnetwithman) and all SNPS in the 184 line data set.  
  This program splits the calculations into those with subsets of these SNPs, which are then reassembled 
 to make the file alldata.ldpi184.  The code that does the assembly is not included.
  The input files are:
   alldata.gof2hq3_05 - included.
   f2highq184invmsked  - not included, generated by SNPdata/readSNPquality20.sas
   f2LDHQ184  - not included, generated by SNPdata/readSNPquality20.sas

This needs to be done carefully,
as the distant correlations need to be with all the SNPs in the 184 (F2highq184mac5), but only with those in the masked set
within inversion breakpoints (F2highq184invmsked).  ;

************************************************;
* Change when moving windows-UNIX;
*libname freeze2 '$HOME/DGRP/data.freeze2';  * Unix form;
*libname freeze2  "D:\Dropbox\DH\Flywing\DGRP\data.freeze2";  * Windows form ;
libname alldata "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data";

************************************************;

/* DATA for this macro
SNP is a unique identifier for the SNP location calculated by adding the FB5 coordinate for the SNP, 
	plus a 10 X 10^9 if the site is on the X chromosome, 21  X 10^9  is on chromosome 2L, etc.  
Chrom		Chromosome arm (not used in the calculations)
snparbnum 	a second unique identifier of SNP.  Just the order in the input file.
psnp184 		Minor allele frequency
pb			the allele frequency bin the SNP is in. 
nlines 		number of lines with non-missing genotypes
nummiss		number of lines with missing genotype
L1-L184  	The genotypes of each line, numbered consecutively.  0 = common homozygote, 1 = rare homozygote, .  is missing value.
		All heterozygotes are scored as missing values.;
Other variables give other useful information not essential to this macro.
*/
  
* Macro datasampsz returns the number of records in a data set.;
%macro datasampsz(dset,returnCode=);
data _null_;
  set &dset nobs=nobs;
 call symput('dn',trim(left(nobs)));
run;
%let returnCode=&dn;
%mend; *datasampsz;  

/* --------------------------------------------- */
/* MACRO GC calculates all correlations within the specified range of minor allele frequencies.
 Parameters are:
cutrsq the cutoff for the minimum r^2 to be saved.
outlibr name of the SAS library for saving results to.
firstp	the number of the first MAF bin whose LD will be calculated. 
	Macro language expects an integer variable, so I divide by 100 to get MAF for the bin. 
lastp  the number of the last MAF bin whose LD will be calculated.
label	a string to add to the output file name
out    an indicator for how severely log output is suppressed. yes=Note creation of all data sets. Anything else =bare minimum log output. 
All Results Viewer output is suppressed. To activate these, remove either of the statements:
 ods html close;	
 ods results off;
or
 ods listing close;

HOWEVER doing so will vastly increase run time, fill hard drives, etc.  
*/
/* ----------------------------------------------  */
data FDR5invmsked;
 length snp 8. psec 4. pbin psnp184 8. Ln1-Ln184 3.;
array f2withdata[184] L21 L26 L28 L38 L40 L41 L42 L45 L57 L59 L69 L73 L75 L83 L85 L88 L91 L93 L101 L105 L109 L129 L136 L138 L142 L149 L153 
L158 L161 L176 L177 L181 L189 L195 L208 L217 L223 L227 L228 L229 L233 L235 L237 L239 L256 L280 L287 L301 L303 L304 L306 L307 L309 L310 L313 L315 
L317 L318 L319 L320 L321 L324 L325 L332 L335 L336 L338 L340 L350 L352 L356 L357 L358 L359 L360 L361 L362 L365 L367 L370 L371 L373 L374 L375 L377 
L379 L380 L381 L382 L383 L385 L386 L391 L392 L399 L405 L406 L409 L426 L427 L437 L439 L440 L441 L443 L461 L486 L491 L492 L502 L508 L509 L513 
L517 L530 L531 L535 L555 L563 L584 L589 L595 L639 L642 L646 L703 L705 L707 L712 L714 L716 L721 L730 L732 L737 L738 L748 L757 L761 L765 L774 L776 
L783 L786 L787 L790 L796 L799 L801 L802 L804 L805 L808 L810 L812 L818 L819 L820 L821 L822 L832 L837 L843 L849 L850 L852 L853 L855 L857 L859 L861 L879 L882 L884 
L887 L890 L892 L894 L897 L900 L907 L908 L911 L913;
	array lcons[184] Ln1-Ln184;
merge alldata.f2highq184invmsked alldata.gof2hq3_05 (IN=A);
 by snp;
 if A;
 pbin = round(psnp184,0.01);
 psec=round(100*pbin);
     do i = 1 to 184;
	 lcons[i]=f2withdata[i];
	end;
 keep snp psec pbin psnp184 mafpi nlines184 nlinespi Ln1-Ln184 ininversion chrom snparbnum;
run;

proc freq; tables pbin;
run;
data FDR5a0;
 set FDR5invmsked;
 where ininversion=0;
 run;
 * 1 is arm 2L, 2 is arm 2R, 3 is arm 3R; 
data FDR5a1;
 set FDR5invmsked;
 where ininversion=1 & snp> 2100000000 & snp<2200000000; 
 psnp184=mafpi;
 nlines184=nlinespi;
 run;
 data FDR5a2;
 set FDR5invmsked;
 where ininversion=1 & snp> 2200000000 & snp<3000000000; 
 psnp184=mafpi;
 nlines184=nlinespi;
 run;
 data FDR5a3;
 set FDR5invmsked;
 where ininversion=1 & snp> 3200000000 & snp<4000000000; 
 psnp184=mafpi;
 nlines184=nlinespi;
 run;

%macro gc(cutrsq,outlibr,firsti,lasti,label,out);
 options nomlogic nomprint nosymbolgen;
* options mlogic mprint symbolgen; 
  %let datetime_start = %sysfunc(TIME()) ;
  %put START TIME: %sysfunc(datetime(),datetime14.);
 %global dn;
%let cutr=%sysevalf(%sysfunc(sqrt(&cutrsq)));
%put Square root of &cutrsq = &cutr;
 %let chunk=100;  * Correlations are calculated in chunks of this many SNPs at a time.;
 %let cchunk=100000;  * for splitting up the comparison calculations.;
 %let err=0;
 %do a=&firsti %to &lasti;
* proc freq data=FDR5a&a noprint;
*  tables psec / out=psecs;
*  run;  
 %*do q = 5 %to 5;  * Major loop over pbins. ;
 %do q = 2 %to 50;  * Major loop over pbins. ;
 * First destroy any old output file of the same name as the output from this run.;  
 * Are you SURE you want to do this?;
 %if %sysfunc(exist(&outlibr..LD&a&label&q)) %then %do;
   proc datasets lib=&outlibr;
	  delete LD&a&label&q;
     quit;
  run;
 %end;
 %let chnk=&chunk;
 * filename logfile "$HOME/dhlustre/DGRP/LD/gcorrsas&label&q..log";  
  filename logfile "D:/Dropbox/DH/Flywing/DGRP/GWAS Programs and data/data/LD&a&label&q..log";
 %if &out^=yes %then %do;
  proc printto log=logfile;
run; 
%end;
%let pbin=%sysevalf(&q/100);
%let ptop = %sysevalf(0.007 + &pbin*1.914-1.268*&pbin**2-0.093*&pbin**3);  * This formula is specific to cutr=0.5, for r^2>0.5.  
		Choice of a different cutoff would be necessary to change formula.;
%let pbot = %sysevalf(-0.00283 + 0.519*&pbin-0.12*&pbin**2+0.359*&pbin**3);
 %put pbin= &pbin ptop= &ptop pbot=&pbot;

 * data set fsnps reads in the focal MAF bin.;
  data fsnps;
   *set freeze2.GCorrExampleData;
   set FDR5a&a;
   where pbin=&pbin;
   * if _N_>200 then stop;  * for debugging, only read in a small number of sites. ;
   rename snparbnum=focalsnp;
   idvar= compress('fsnp'||_N_);
   run;
%datasampsz(fsnps);
%let nfsnps=&dn;
%if &nfsnps>0 %then %do;
	* data set csnps contains all the sites close in frequency to the focal SNPs that can yield a correlation
	  of r^2>0.5 with those in the focal bin.;
	  %if &a=0 %then %do;
	  data csnps;
	   set alldata.f2LDHQ184;
	   *where psnp184>0.28 and psnp184<=0.29;
	   where psnp184>&pbot and psnp184<=&ptop;
	   *  if _N_>100000 then stop; * for debugging, only read in a small number of sites. ;
	   rename snparbnum=compsnp;
	  run;
	  %end;
	  %if &a=1 %then %do;
	    * In2lt region.;
	  data wsnps;
	   set alldata.f2LDHQ184;
	   *where psnp184>0.28 and psnp184<=0.29 and ((ininversion=1 and chrom^='2L') or ininversion=0);
	   where psnp184>&pbot and psnp184<=&ptop and ((ininversion=1 and chrom^='2L') or ininversion=0);
	   rename snparbnum=compsnp;
	  run;
	  data isnps;
	    set alldata.f2ldhq184invmsked;;
	     *where psnp184>0.28 and psnp184<=0.29 and (ininversion=1 and chrom='2L');
	     where psnp184>&pbot and psnp184<=&ptop  and (ininversion=1 and chrom='2L');
		 run;
	  proc freq;
	   tables chrom*ininversion;
	   run;
	   data csnps;
	    set wsnps isnps;
		run;
		 proc sort;
		  by snp;
		  run;
      %end;
	  %if &a=2 %then %do;
	    * In2RNS region.;
	  data wsnps;
	   set alldata.f2LDHQ184;
	   * where psnp184>0.28 and psnp184<=0.29 and ((ininversion=1 and chrom^='2R') or ininversion=0);
	   where psnp184>&pbot and psnp184<=&ptop and ((ininversion=1 and chrom^='2R') or ininversion=0);
	   rename snparbnum=compsnp;
	  run;
	  data isnps;
	    set alldata.f2ldhq184invmsked;;
	    * where psnp184>0.28 and psnp184<=0.29 and (ininversion=1 and chrom='2R');
	    where psnp184>&pbot and psnp184<=&ptop  and (ininversion=1 and chrom='2R');
		 run;
	   data csnps;
	    set wsnps isnps;
		run;
		 proc sort;
		  by snp;
		  run;
	  %end;
	  %if &a=3 %then %do;
	    * In3RMo region.;
	  
	  data wsnps;
	   set alldata.f2LDHQ184;
	   *where psnp184>0.28 and psnp184<=0.29 and ((ininversion=1 and chrom^='3R') or ininversion=0);
	   where psnp184>&pbot and psnp184<=&ptop and ((ininversion=1 and chrom^='3R') or ininversion=0);
	   rename snparbnum=compsnp;
	  run;
	  data isnps;
	    set alldata.f2ldhq184invmsked;
	   * where psnp184>0.28 and psnp184<=0.29 and (ininversion=1 and chrom='3R');
	   where psnp184>&pbot and psnp184<=&ptop  and (ininversion=1 and chrom='3R');
		 run;
	   data csnps;
	    set wsnps isnps;
		run;
		 proc sort;
		  by snp;
		  run;
	  %end;
	  data csnps;
	   set csnps;
	   * if _N_>100000 then stop;
		idvar= compress('csnp'||_N_);
		keep snp idvar psnp184 nlines184 Ln1-Ln184 ininversion chrom compsnp;
	   run;
	 %datasampsz(csnps);
	%let ncsnps=&dn; 
	* This macro code will fill up any hard drive with output unless output is suspended: 
	 These commands turn off results output to Output window, and Results Viewer.;
	 ods listing close; * Turns off Output to Output window.;  
	  ods html close;  * These two commands turn off the Results Tracking and Results viewer output.;
	 ods results off;
 	* Transpose the data before calculating correlations.;
*	proc transpose data=fsnps prefix=fsnp out=ftsnp; 
*	   var Ln1-Ln184;
*	run;


	 %let nch=%eval((&nfsnps-1)/&chunk+1);  * For the chunk value chosen above, how many chunks will it take to go through the whole bin?;
	*** Now get the data for SNPs with allele frequencies greater than but close to the bin for ftsnp. output gwgcorr;
	 %let chnk=&chunk;
	 %let cchnk=&cchunk;

	%if &ncsnps>0 %then %do; *****************************
		  proc printto;
		  run;
		%put Focal vs. comparison set &a pbin &pbin &nfsnps &ncsnps %sysfunc(datetime(),datetime18.) ;
			 %if &out^=yes %then %do;
			  proc printto log=logfile;
			  run; 
			%end;
		* Preparing data.;

		data fcc;
		 set fsnps csnps;
		run;
		proc transpose data=fcc out=fc;
		  var Ln1-Ln184;
		  id idvar;
		  run;
	* This code essentially repeats the above, but the correlations are between SNPs in focal MAF bin and the SNPs outside the bin.;
		 %let nch=%eval((&nfsnps-1)/&chunk+1);
		 %let ncch=%eval((&ncsnps-1)/&cchunk+1);
		 %do r=1 %to &nch;
		 %let firstf=%eval((&r-1)*&chunk+1);
		 %if &r=&nch %then %do;
				%let lastf=&nfsnps;
				%let chnk=%eval(&lastf-&firstf+1);
			%end;
		 %else %let lastf=%eval(&firstf+&chunk-1);
		    %*do c=1 %to 4;
		    %do c=1 %to &ncch;
				 %let cfirstf=%eval((&c-1)*&cchunk+1);
				 %if &c=&ncch %then %do;
						%let clastf=&ncsnps;
						%let cchnk=%eval(&clastf-&cfirstf+1);
					%end;
				 %else %let clastf=%eval(&cfirstf+&cchunk-1);
			  proc printto;
			  run;
			 %put  Focal vs. comparison chunking: set &a p=&pbin,  &nfsnps with &ncsnps snps. &r of &nch chunks (&firstf to &lastf.), &c of &ncch chunks (&cfirstf to &clastf.). %sysfunc(datetime(),datetime18.);
			 %if &out^=yes %then %do;
			  proc printto log=logfile;
			run; 
			 %put  Focal vs. comparison chunking: set &a p=&pbin,  &nfsnps with &ncsnps snps. &r of &nch chunks (&firstf to &lastf.), &c of &ncch chunks (&cfirstf to &clastf.). %sysfunc(datetime(),datetime18.);
			%end;
			 ods listing close;
			* Now calculate correlations between SNPs in focal MAF bin and the SNPs outside the bin.;
				ods output PearsonCorr=fccorrc;
				proc corr noprob data=fc;
				 var fsnp&firstf-fsnp&lastf;
				 with csnp&cfirstf-csnp&clastf;
				run;
				ods output close;
				ods listing;

			*  proc printto print='d:\flywing\dgrp\gwcorrdump.lst';
			* run;
			proc iml;
			 * output has default width of 15;
			  reset fw=15;
			  use fccorrc;
			   read all var {
			%do i = &firstf %to &lastf;
			   fsnp&i
			%end;
			} into corr;
			   read all var {
			%do i = &firstf %to &lastf;
			   nfsnp&i
			%end;
			} into corrn; 
			   close fccorrc; 

			    use fsnps;
			     read all var {focalsnp snp nlines184 psnp184} into focalsnp;
			   close fsnps;
			     use csnps;
			     read all var {compsnp snp nlines184 psnp184} into compsnp;
			   close csnps;
			   Bigs=loc(corr>&cutr);
				nhigh=ncol(Bigs); 
			   if nhigh>0 then do; * There are some high correlations.;
				   arow=floor((Bigs-1)/&chnk)+1;
				   column = &firstf -1 + ( (Bigs-&chnk*(arow-1)) );
				   row=&cfirstf-1 +arow;
				   dw = corr[Bigs];
				  dn = corrn[Bigs];
				  * print (Bigs`) (row`) (arow`) (column`) dw dn;
				  mr = max(row);
				  if mr>nrow(compsnp) then do;
					print "focal vs. comparison in &q, &r of &nch, from &firstf to &lastf..";
				    mbig=max(Bigs);
				    print nhigh mr mbig;
					print Bigs;
					colrow=row//col;
					print colrow;
					call symput('err','1');
				  end;
				  fid = focalsnp[column,]; * This is where there is an error;
				  cid= compsnp[row,];
					corres=fid||cid||dn||dw;
				 	create gwgcorr from corres[colname={arb1 snp1 snp1n psnp1 arb2 snp2 snp2n psnp2 incommon dcorr}];
				     append from corres;
					close gwgcorr; 
				end;
				else do;
				  * No high correlations are detected in this chunk, so delete the old gwgcorr file, so it doesnt get 
						appended twice.;
				  %if %sysfunc(exist(gwgcorr)) %then %do; 
					call delete(gwgcorr);
				  %end;
				  print 'No high correlations.';
				end;
			**** done with the focal vs. comparison calculations.;

			quit;
			%if &err=1 %then %do;
			%put error exiting with &firstf &lastf;
				%goto exit;
			%end; 
			 ods listing close;
			 %if %sysfunc(exist(gwgcorr)) %then %do;
			 	%datasampsz(gwgcorr);
				%let nco=&dn;
				  %if %sysfunc(exist(&outlibr..LD&a&label&q)) %then %do;
					   proc datasets;
						  append base=&outlibr..LD&a&label&q data=gwgcorr;
						quit;
						  run;
				  %end;
				  %else %do;
					  data &outlibr..LD&a&label&q (label='LD correlations with significant HQ SNPS, freeze 2 data. Organized by p similarity.');
					   set gwgcorr;
					  run;
				  %end;
			  %end;
			  proc printto;
			  run;
			  %put %sysfunc(datetime(),datetime18.);
				 %if &out^=yes %then %do;
					  proc printto log=logfile;
					run; 
					%end;
			%end;  * c chunks of csnps;
		  %end; *r;
		 %if &out^=yes %then %do;
		  dm'odsresults; clear';
		  proc printto;
		  run;
		%end; 
	  %end;  * There are csnps with higher frequencies.; 
	%end; * There are fsnps in this bin;
%end; * q; 
%end; * a;
 %exit: ods listing;
  ods html;
  ods results on;
  proc printto;
  run;

 %put END TIME: %sysfunc(datetime(),datetime14.);
 %put PROCESSING TIME:  %sysfunc(putn(%sysevalf(%sysfunc(TIME())-&datetime_start.),mmss.)) (mm:ss) ;
%mend;

* indices for starting and ending p bins, label.;
%*macro gc(cutr,outlibr,firsti,lasti,label,out);

%gc(0.5,alldata,1,3,f2sig184hq,no);
 
