/* This file is part of the code used to analyze data for 
Pitchers et al. 2018 (or 2019), A multivariate genome-wide association 
study of wing shape in Drosophila melanogaster to be published in Genetics. */

* Summarizing the amount of long-distance disequilbrium for significant SNPs in the 184 lines.;
* Run after CHCorrHQ184sig.sas ;

ODS HTML CLOSE;
  ODS HTML;
  libname alldata "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data";
options sasautos=('D:\Dropbox\DH/Gmatrixanalysis/SASMacros');

 * file with snp genotypes and maf;
 data snp;
   set alldata.F2highq184invmsked;
    trp=int(100*psnp184);
  keep snp psnp184 mac184 pbin184 nummiss184 nlines184 trp;
run;
data snpp;
 merge alldata.gof2hq3_05(in=A) snp;
 by snp;
 if A;
  altpbin=round(psnp184,0.025);
  keep snp psnp184 mac184 pbin184 nummiss184 nlines184 altpbin;
 run;
proc freq data=snpp;
 tables altpbin;
 run;
 

proc means mean median n data=alldata.F2highq184invmsked(where=(snp>5 & mac184>=5));
 var nummiss184 nlines184 mac184 psnp184;
run; 

options ls=150 nolabel;

%macro diffchrom(minrsq,Rsq);
%let minr=%sysevalf(%sysfunc(sqrt(&Rsq)));
	data Dmc&minrsq;
	  set alldata.ldpi184;
	  where dcorr>=&minr;
	  *if _N_>100000 then stop;
	  * Now make special adjustments for the inversions.;
	  if snp1<4 then do;
	   select (snp1);
		   when (1) dist=abs(snp1+2100000000-snp2);
		   when (2) dist=abs(snp1+2200000000-snp2);
		   when (3) dist=abs(snp1+3200000000-snp2);
		   otherwise;
	 	 end;
	  end;
      else if snp2<4 then do;
	    select(snp2);
	  	   when (1) dist=abs(snp2+2100000000-snp1);
		   when (2) dist=abs(snp2+2200000000-snp1);
		   when (3) dist=abs(snp2+3200000000-snp1);
		   otherwise;
	     end;
	  end;
	  else dist = abs(snp1-snp2);
	  if dist>130000000 then difchr=1;
	  else difchr=0;
	  if dist>100000 then far=1;
	  else far=0;
	run;
	
	* Adding up frequencies of correlations by distance classes.;
	proc freq data= Dmc&minrsq;
	 tables snp1 / noprint out=numhighc&minrsq;
	run;
	data numhighc&minrsq;
	 set numhighc&minrsq;
	 rename count=numhighc;
	run;
	proc freq data= Dmc&minrsq;
		tables snp1 /  noprint out=numdifchr&minrsq;
     	weight difchr;
	 run;
	data numdifchr&minrsq;
	 set numdifchr&minrsq;
	 rename count=numdifchr;
	run;
	proc freq data= Dmc&minrsq;
		tables snp1 /  noprint out=numfar&minrsq;
     	weight far;
	 run;
	data numfar&minrsq;
	 set numfar&minrsq;
	 rename count=numfar;
	run;
	data corrdist&minrsq;
	 merge numhighc&minrsq numfar&minrsq numdifchr&minrsq;
	 by snp1;
	 if numfar=. then numfar=0;
	 if numdifchr=. then numdifchr=0;
	 rename snp1=snp;
	 run;
/* 
 	proc freq data= corrdist&minrsq;
	 tables snp / noprint out=numhighc&minrsq;
	 weight numhighc;
	run;
	data numhighc&minrsq;
	 set numhighc&minrsq;
	 rename count=numhighc;
	run;
	proc freq data= corrdist&minrsq;
		tables snp /  noprint out=numdifchr&minrsq;
     	weight numdifchr;
	 run;
	data numdifchr&minrsq;
	 set numdifchr&minrsq;
	 rename count=numdifchr;
	run;
	proc freq data= corrdist&minrsq;
		tables snp /  noprint out=numfar&minrsq;
     	weight numfar;
	 run;
	data numfar&minrsq;
	 set numfar&minrsq;
	 rename count=numfar;
	run;
	data allcorrsum&minrsq;
	 merge numhighc&minrsq numfar&minrsq numdifchr&minrsq;
	 by snp;
	 if numfar=. then numfar=0;
	 if numdifchr=. then numdifchr=0;
	 run;
 proc sort data=allcorrsum&minrsq;
  by snp;
 run;  */

 data alldata.corrcounts184&minrsq
(label='Counts of correlations with r^2>0.5 for significant snps, inversions. Generated by macro diffchrom in LinkageDiseq/hqcountsigcorrs184.sas');
   merge corrdist&minrsq snpp(in=A);
   by snp;
   if A;
   	 if numfar=. then numfar=0;
	 if numdifchr=. then numdifchr=0; 
	 if numhighc=. then numhighc=0;
	rename numhighc=numhighc&minrsq numfar=numfar&minrsq numdifchr=numdifchr&minrsq;
	drop percent;
  run;
%mend;

%diffchrom(5,0.5);

proc means mean median n data=alldata.corrcounts1845(where=(snp>5));
 var numhighc5 numfar5 numdifchr5 mac184 psnp184;
run; 
%macro calcthings(minrsq,inv);
data cc&minrsq&inv;
 format chrompart $12.;
 set alldata.corrcounts184&minrsq;
 where snp>5;
 if numhighc&minrsq>1 then somer=1; else somer=0;
 if numfar&minrsq>1 then somefar=1; else somefar=0;
 if numdifchr&minrsq>1 then somedc=1; else somedc=0;
	  select;
	   when (snp<2000000000) chrompart='X';
	   when (snp<2102215744) chrompart='2Lprox';
	   when (snp<2102235744) chrompart='2Lproxbreak';
	   when (snp<2113144180) chrompart='2LIn(2L)t';
	   when (snp<2113164180) chrompart='2Ldistrbreak';
	   when (snp<2200000000) chrompart='2Ldist';
	   when (snp<2211268659) chrompart='2Rprox';
	   when (snp<2211288659) chrompart='2Rproxbreak';
	   when (snp<2216153839) chrompart='2RIn(2R)NS';
	   when (snp<2216173839) chrompart='2Rdistrbreak';
	   when (snp<3000000000) chrompart='2Rdist';
	   when (snp<3121300000) chrompart='3Ldist';
	   when (snp<3200000000) chrompart='3Lprox';
	   when (snp<3217242639) chrompart='3Rprox';
	   when (snp<3217262639) chrompart='3Rproxbreak';
	   when (snp<3224847019) chrompart='3RIn(3R)MO';
	   when (snp<3224867019) chrompart='3Rdistrbreak';
       when (snp<4000000000) chrompart='3Rdist';
	   otherwise chrompart='4';
	   end;
run;
%if &inv eq noinv %then %do;
   data cc&minrsq&inv;
    set cc&minrsq&inv;
*	where index(chrompart,'In(')=0 & index(chrompart,'break')=0;
	* including the breakpoint region makes very little difference.;
 where index(chrompart,'In(')=0 & index(chrompart,'3Rdist')=0;
   run;
%end;
%if &inv eq inv %then %do;
   data cc&minrsq&inv;
    set cc&minrsq&inv;
*	where index(chrompart,'In(')=0 & index(chrompart,'break')=0;
	* including the breakpoint region makes very little difference.;
 where index(chrompart,'In(')>0 | index(chrompart,'3Rdist')>0;
   run;
%end;
title "Correlations with R^2>0.&minrsq.. Inversions=&inv";
proc freq;
  tables somer somefar;
  run;
proc freq;
 tables somer*altpbin / out=freqall&inv;
 tables  somefar*altpbin / out=freqfar&inv;
 tables somedc*altpbin / out=freqdc&inv;
 run;
proc univariate data=cc&minrsq&inv;
 var numhighc&minrsq numfar&minrsq numdifchr&minrsq;
 run;
proc means data=cc&minrsq&inv mean median max min;
 var numhighc&minrsq numfar&minrsq numdifchr&minrsq;
 run;
 proc sort data=cc&minrsq&inv;
  by altpbin;
run;
proc means data=cc&minrsq&inv noprint mean median max min;
 var numhighc&minrsq numfar&minrsq numdifchr&minrsq;
 output out=alldata.hqmccount184&inv&minrsq(label='Statistics for correlations with r^2>0.5 for all snps, inversions. Generated by macro calcthings in freeze2/gccountcorrsf2184.sas')
 mean= mhigh&minrsq mfar&minrsq mdc&minrsq median= medhigh&minrsq medfar&minrsq meddc&minrsq 
  min= minhigh&minrsq minfar&minrsq mindc&minrsq 
  max= maxhigh&minrsq maxfar&minrsq maxdc&minrsq;
 by altpbin;
run;
PROC EXPORT DATA= alldata.hqMccount184&inv&minrsq 
            OUTFILE= "D:\Dropbox\DH\Flywing\DGRP\Highquality\HQmccount184&inv&minrsq..txt" 
            DBMS=TAB REPLACE;
     PUTNAMES=YES;
 run; 
 %mend;
* r^2 label, then is this leaving out inversions (noinv) or not (inv) or has everything (all);
%calcthings(5,all);
%calcthings(5,inv);
%calcthings(5,noinv);

proc freq data=cc5all;
 tables somer somefar somedc;
 run;
proc freq data=cc5all(where=(psnp184>0.1 and psnp184<0.2));
 tables somer somefar somedc;
 run;
option nolabel;
data reall;
 merge freqallall(where=(somer=0) rename=(count=allnotcount)) freqallall(where=(somer=1) rename=(count=allcorrcount) )
  freqallinv(where=(somer=0) rename=(count=invnotcount)) freqallinv(where=(somer=1) rename=(count=invcorrcount))
 freqallnoinv(where=(somer=0) rename=(count=noinvnotcount)) freqallnoinv(where=(somer=1) rename=(count=noinvcorrcount));
 by altpbin;
 allprop=allcorrcount/(allnotcount+allcorrcount);
 invprop=invcorrcount/(invnotcount+invcorrcount);
 noinvprop=noinvcorrcount/(noinvnotcount+noinvcorrcount);
run;
data refar;
 merge freqfarall(where=(somefar=0) rename=(count=allnotfarc)) freqfarall(where=(somefar=1) rename=(count=allcorrfarc) )
  freqfarinv(where=(somefar=0) rename=(count=invnotfarc)) freqfarinv(where=(somefar=1) rename=(count=invcorrfarc))
 freqfarnoinv(where=(somefar=0) rename=(count=noinvnotfarc)) freqfarnoinv(where=(somefar=1) rename=(count=noinvcorrfarc));
 by altpbin;
 allfarprop=allcorrfarc/(allnotfarc+allcorrfarc);
 if invcorrfarc>0 then invfarprop=invcorrfarc/(invnotfarc+invcorrfarc);
 else invfarprop=0;
 noinvfarprop=noinvcorrfarc/(noinvnotfarc+noinvcorrfarc);
run;
data propcorre; 
length  allprop invprop noinvprop allfarprop invfarprop  noinvfarprop 8.;
  array x[*] allprop invprop noinvprop allfarprop invfarprop  noinvfarprop allnotcount allcorrcount invnotcount invcorrcount noinvnotcount noinvcorcount allnotfarc allcorrfac 
 invnotfarc invcorrfarc noinvnotfarc noinvcorrfarc allcorrfarc noinvcorrcount;
 merge reall refar;
 by altpbin;
 do i = 1 to dim(x);
  if x[i]=. then x[i]=0;
  end;
 drop somefar percent somer i;
 run;

data invnoinvmeans;
length altpbin logallmhigh5 logallmfar5 allmedhigh5 lognoinvmhigh5 lognoinvmfar5 noinvmedhigh5 loginvmhigh5 
  loginvmfar5 invmedhigh5 allprop invprop noinvprop allfarprop invfarprop  noinvfarprop 8.;
  array n[*] allmhigh5 allmfar5 invmhigh5 invmfar5 noinvmhigh5 noinvmfar5;
  array l[*] logallmhigh5 logallmfar5 loginvmhigh5 loginvmfar5 lognoinvmhigh5 lognoinvmfar5;
  merge alldata.Hqmccount184all5(rename=(mhigh5=allmhigh5 mfar5=allmfar5 medhigh5=allmedhigh5))
	alldata.Hqmccount184inv5(rename=(mhigh5=invmhigh5 mfar5=invmfar5 medhigh5=invmedhigh5)) 
  alldata.Hqmccount184noinv5(rename=(mhigh5=noinvmhigh5 mfar5=noinvmfar5 medhigh5=noinvmedhigh5)) propcorre; 
  by altpbin;
  do i= 1 to dim(n);
   if n[i]>0 then l[i]=log10(n[i]);
   else l[i]=-0.5;
   end;
run;

PROC EXPORT DATA= WORK.invnoinvmeans 
            OUTFILE= "D:\Dropbox\DH\Flywing\DGRP\Highquality\corrsig184.txt" 
            DBMS=TAB REPLACE;
     PUTNAMES=YES;
RUN;

proc sort data=chrsectsmillionLD;
by chrom;
 run;


 * Now investigating SNP density;
 data somesnp;
 set alldata.f2ldhqdata;
 where mac184>4;
* List of all 184 lines.;
 snparbnum=_N_;
section=round(SNP,1000000)/1000000;
 run;

proc freq data=somesnp;
 tables pb chrom pb*chrom;
run; 
proc freq data=somesnp noprint;
  tables chrom*section / out=snpsect;
run;
proc sort data=snpsect;
by chrom;
 run;
proc plot data=snpsect(where=(section>0));
plot count*section;
by chrom;
run;

* For graphing the amount of LD;
data cccorrdensity;
 set alldata.corrcounts1845;
 format chrom $2.;
 if pbin184>50 then do;
   psnp=1-psnp;
   pbin184 = round(100*psnp);
end;
section=round(SNP,100000)/100000;
 	  select;
	   when (SNP<2000000000) do;
         chrom='X';
		 gpos=section-10000+1;
		 end;
	   when (SNP<2200000000) do;
		chrom='2L';
		 gpos=section-21000+1;
		 end;
	   when (SNP<3000000000) do;
		chrom='2R';
		 gpos=section-22000+1;
		 end;
	   when (SNP<3200000000) do;
 			chrom='3L';
		 gpos=section-31000+1;
		 end;
	   when (SNP<4000000000) do;
     chrom='3R';
		 gpos=section-32000+1;
		 end;
	   otherwise do;
			chrom='4';
		 gpos=section-40000+1;
		 end;
	   end;   
run;
proc sort data=cccorrdensity;
 by chrom gpos;
run;
proc means data=cccorrdensity noprint mean;
 var numhighc5 numfar5 numdifchr5;
 output out=chrsects100kLD mean= mhigh mfar mdc;
 by chrom gpos;
run;
PROC EXPORT DATA= WORK.Chrsects100kld 
            OUTFILE= "D:\Dropbox\DH\Flywing\DGRP\LDpaper\hqgenomehighLD.csv" 
            DBMS=CSV REPLACE;
     PUTNAMES=YES;
RUN;

* Getting distribution of r^2 values above 0.5;

%macro distrsq(minrsq,Rsq);

*options symbolgen mprint mlogic;
options nomlogic nomprint nosymbolgen;
  %let mincorr=%sysfunc(sqrt(%sysevalf(&Rsq)));
  %put R &mincorr;
 
/* 	data Dmc&minrsq;
	  set alldata.ld184allunique;
	  where dcorr>&mincorr & snp1>5 & snp2>5;  
      hist=round(20*(dcorr**2-0.025));
	run;
	proc freq;
	  tables hist / out=histcorrd&minrsq;
	run; */
 data alldata.corrdhist184&minrsq
(label='Data for histogram of values of high r^2, r^2>0.5 for all snps. Generated by macro distrsq in freeze2/gccountcorrsf2184.sas');
   set histcorrd&minrsq;

  run;
%mend;

%distrsq(5,0.5);

proc export data=alldata.corrdhist1845
 outfile="D:\Dropbox\DH\Flywing\DGRP\LDpaper\hqcorrhistogram.csv"
 DBMS=CSV REPLACE;
 Putnames=Yes;
 run;

