/* This file is part of the code used to analyze data for 
Pitchers et al. 2018 (or 2019), A multivariate genome-wide association 
study of wing shape in Drosophila melanogaster to be published in Genetics. */

/* This SAS program generates the method of moments MANOVA results that are then subjected to 
  the FDR calculations to choose the 2396 SNPS for further study. 
  The input files are:
	phenotype data in DGRP_Wing_Data_HouleDworkin.csv  - This program then makes the SAS data set data/interlabsizejun.sas7bdat from these data
    Genotype data in SAS data set f2highq184invmsked, which is NOT included with this distribution.  
    	To generate f2highq184invmsked, run the furnished program SNPdata\ReadSNPquality20.sas. This also requires 
		obtaining the DGRP data as instructed in that file. 

	This program was rewritten March 2015 for the high quality results based on the May 2013 version.
	This is a Windows version, but most results were obtained on Linux. */

/* All effects are considered in the estimation of error matrices, but only SNP effect is tested */
/* This program analyses the combined data sets from the Houle and Dworkin labs.  
* This version calculates means by line, lab and sex as data. ;*/
libname alldata "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data";

Proc IMPORT OUT= WORK.alldata.interlabsizejun 
            DATAFILE= "D:\Dropbox\DH\Flywing\DGRP\GWAS Programs and data\data\DGRP_Wing_Data_HouleDworkin.csv" 
            DBMS=CSV REPLACE;
     GETNAMES=YES;
	 Guessingrows=25000;
     DATAROW=2; 
RUN;

proc sort data=alldata.interlabsizejun;
 by linenum lab sex block;

proc means data=alldata.interlabsizejun noprint;
 by linenum lab sex;
 var Prin1-Prin58 logcs;
 output out=llsm mean=llsPrin1-llsPrin58 logcs;
 run;
* Standardize the fixed sex and lab effects to mean 0.;
proc sort data=llsm;
  by lab sex;
run;
proc standard data=llsm mean=0 out=llsmean;
 var llsPrin1-llsPrin58 logcs;;
 by lab sex;
run;
proc sort data=llsmean;
 by linenum lab sex;
 run;
data testlabel;
 input effect $;
 cards;
 snp
 ; 
 run;

title 'With second univariate test.';
%macro testSNP(firstchunk,lastchunk,maxprn,libr,label,output);
options nomlogic nomprint nosymbolgen;
* options mlogic mprint symbolgen;
 %let datetime_start = %sysfunc(TIME()) ;
  %put START TIME: %sysfunc(datetime(),datetime14.);
data snpstoan;
  length snp chunk 8.;
  set alldata.f2highq184invmsked;
  where snp>5 & chunk>=&firstchunk & chunk<=&lastchunk;
  * if _N_>2 then stop;
run;
  %do c = &firstchunk %to &lastchunk; 
  data snpstoanalyze;
    set snpstoan;
	where chunk=&c;
	n=_N_;
  run;
%let numvaranl=%eval(&maxprn+1);

data snpsnbtoan;
   set snpstoanalyze;
   *where n>&lastsnp;
    if _N_>10 then stop;
   drop lref lalt;
run;  


%put %sysfunc(datetime(),datetime18.);
%if &output^=out %then %do;
filename logfile "D:\Flywing\DGRP\data.freeze2\HighQ\m&firstchunk.to&lastchunk&label..log";
*filename logfile "/panfs/storage.local/genacc/home/emarquez/lustre/DGRP/Freeze2/InvManova/m&firstchunk.to&lastchunk&label..log";
  proc printto log=logfile;
run; 
options nosource nonotes;
 ods html close;  * These two commands turn off the Results Tracking and Results viewer output.;
 ods results off;
ods listing close;
%end;


proc contents data=snpsnbtoan out=nnbsnps;
data nnbsnps;
 set nnbsnps; * Makes a macro variable called nnsps from a data set variable nobs.;
 call symput('nsnps',nobs);
run;

data snpinfos;
run;

%do i=1 %to &nsnps;
data onesnpnb;
 set snpsnbtoan (firstobs=&i obs=&i);
 * call symput('invid',chrompart);
run;
data snpinfo;
 set onesnpnb;
 snpid=symget('i');
 drop L:;
run;

proc transpose data=onesnpnb out=linesnpnb;
  var L:;
run;
data linesnpnb;
 set linesnpnb;
 rename col1=snp;
 linenum=substr(_NAME_,2)+0;
run;
proc sort;
 by linenum;
run;

* For the model with line-lab means.;
data snpshapenb;
 merge linesnpnb llsmean(in=a);
  by linenum;
  if a;
  rename linenum=line;
run;

data snpinfos;
 set snpinfos snpinfo;
run;

data snpshapenb;
  set snpshapenb;
  if snp=. then delete;
  if _freq_=. then delete;
run;

* Filters by MAC and missing data;

/*%if &onemac>4 and &onemaf<1 and &onensnp>60 %then %do;*/

* GLM1. This univariate run of GLM is to get the weights used to construct the denominator MS.;
ods output RandomModelANOVA=ranmodnb;
proc glm data=snpshapenb; 
 class snp sex lab line;
 *if you change the model and random statements, correct the iml code for reading df!;
 model llsPrin1 llsPrin2=snp sex*snp lab*snp sex lab sex*lab line(snp)  sex*line(snp) lab*line(snp);
* model llsPrin1 =snp sex*snp lab*snp sex lab sex*lab line(snp)  sex*line(snp) lab*line(snp);
random line(snp) sex*line(snp) lab*line(snp) /	test;
  weight _FREQ_;
 run;
ods output close;
quit;
run;

/* %put glm has run %sysfunc(datetime(),datetime18.);*/

* The next section of code is for reading back the weights for the SSCP matrices;
/*
%let snpXsexls=0; 	
%let snpXsexsls=0;  	
%let snpXsexlls=0;
%let snpXsexerr=0;
%let snpXlabls=0;  
%let snpXlabsls=0;
%let snpXlablls=0;
%let snpXlaberr=0;
*/
%let snpls=0; 
%let snplls=0; 
%let snpsls=0;
%let snperr=0; 
option spool;

**** ;
* This data step parses the definition of the denominator MS into the weights for each type.
* It creates macro variables for each weight to carry forward to the IML step.;
* It also saves the weights in the data set for future reference.;
data ranmodnb;
 set ranmodnb;
 source=upcase(source);
 error=upcase(error);
run;
data coeffsnb;
 attrib error length=$120; 
  format error $120.;
 set ranmodnb;
  where Source='SNP' & dependent='llsPrin1'; /*or Source='SNP*SEX' or Source='SNP*LAB';*/
  call symput('PvalPrin1',ProbF);
 done=0;
 sign=1;
 remains=error;
 nextstar = index(remains,'*'); 
 do until (done=1);
    wcoeff= (compress(scan(remains,1,'*'))+0)*sign;
	remains = substr(remains,nextstar+1);
	nword =compress(scan(remains,1,'+-'));
	 select (Source);
	 /*
  		when ('SNP*SEX') do;
	        if index(nword,'SEX*LINE(SNP)')>0 then call symput('snpXsexsls',wcoeff);
	         else if index(nword,'LAB*LINE(SNP)')>0 then call symput('snpXsexlls',wcoeff);
	         else if index(nword,'LINE(SNP)')>0 then call symput('snpXsexls',wcoeff);
	        else if index(nword,'ERROR')>0 then call symput('snpXsexerr',wcoeff);
	    end;
	 */
		when ('SNP') do;
	        if index(nword,'SEX*LINE(SNP)')>0 then call symput('snpsls',wcoeff);
	         else if index(nword,'LAB*LINE(SNP)')>0 then call symput('snplls',wcoeff);
	         else if index(nword,'LINE(SNP)')>0 then call symput('snpls',wcoeff);
	        else if index(nword,'ERROR')>0 then call symput('snperr',wcoeff);
	    end;
	/*
		when ('SNP*LAB') do;
	        if index(nword,'SEX*LINE(SNP)')>0 then call symput('snpXlabsls',wcoeff);
	         else if index(nword,'LAB*LINE(SNP)')>0 then call symput('snpXlablls',wcoeff);
	         else if index(nword,'LINE(SNP)')>0 then call symput('snpXlabls',wcoeff);
	        else if index(nword,'ERROR')>0 then call symput('snpXlaberr',wcoeff);
	    end;
	*/
		otherwise;
		end;
  nextstar = index(remains,'*'); 
  if nextstar=0 then done=1;
  else do;
    nextsign = indexc(remains,'+-'); 
	signn = substr(remains,nextsign,1);
	remains=substr(remains,nextsign+1);
	nextstar = index(remains,'*'); 
	if signn='-' then sign=-1;
	else sign=1;
	end;
 end;  * loop for parsing;
 run;

 /*
 *This section of code checks that the weight variables saved in macro variables sum to 1, and to alert the user if they dont;
%let coefOK=1;
   %let coeftest2=%sysevalf(&snpXsexls+&snpXsexsls+&snpXsexlls+&snpXsexerr);
  %if &coeftest2<0.95 | &coeftest2>1.05 %then %let coefOK=0;
  %let coeftest3=%sysevalf(&snpXlabls+&snpXlabsls+&snpXlablls+&snpXlaberr);
  %if &coeftest3<0.95 | &coeftest3>1.05 %then %let coefOK=0;
  %let coeftest1=%sysevalf(&snpls+&snplls+&snpsls+&snperr);
  %if &coeftest1<0.95 | &coeftest1>1.05 %then %let coefOK=0; 
  %put &coefOK;
  %if &coefOK=0 %then %do;
    %put coefficients do not add up;
    %put snp &snpls &snplls &snpsls &snperr &coeftest1;
    %put snpXsex &snpXsexls &snpXsexsls &snpXsexlls &snpXsexerr &coeftest2;
    %put snpXlab &snpXlabls &snpXlabsls &snpXlablls &snpXlaberr &coeftest3;
  %end; 
 */
data Pval2;
 set ranmodnb;
 where Source='SNP' & dependent='llsPrin2';
  call symput('PvalPrin2',ProbF);
run; 

******; 
* GLM 2. This is the run of GLM that actually calculates the SSCP matrices. The SSCP matrices are saved in the data set sshnb.;
proc glm data=snpshapenb outstat=sshnb; 
 class snp sex lab line;
 model llsPrin1-llsPrin&maxprn logcs=sex sex*snp sex*lab lab|snp line(snp)  sex*line(snp) lab*line(snp) / nouni;
 random line(snp) sex*line(snp) lab*line(snp);
 weight _FREQ_;
 run;
quit;
run;
data sshnb;
 set sshnb;
  _SOURCE_=upcase(_SOURCE_);
run;

/* %put Done 2nd GLM %sysfunc(datetime(),datetime18.);*/

* GLM 3. This run of GLM just gets the least squares means for each snp, for calculation of effect sizes.;
* I think this separate step is necessary because the lsmeans are non-estimable in the more complex model fit in GLM 2.;
ods output lsmeans=nblsmeans;
proc glm data=snpshapenb; 
 class snp sex lab line;
 model llsPrin1-llsPrin&maxprn logcs=sex sex*lab lab|snp line(snp) / nouni;
 random line(snp);
  weight _FREQ_;
  lsmeans snp;
 run;
ods output close;
quit;
run;

/*%put Done 3rd GLM %sysfunc(datetime(),datetime18.);*/

/*
proc sort data=snpshapenb;
 by sex;
run;
ods output lsmeans=nblsmeanbysex;
proc glm data=snpshapenb; 
 class snp lab line;
 model llsPrin1-llsPrin&maxprn logcs= lab|snp line(snp) / nouni;
 random line(snp);
  weight _FREQ_;
  lsmeans snp;
  by sex;
 run;
ods output close;
quit;
run;
*/

/* %put Done by sex GLM %sysfunc(datetime(),datetime18.);*/

* IML is now run to weight the SSCP matrices by the coefficients estimated in GLM run 1,
then calculate Wilks lambda and other statistics for the MANOVA.;
proc iml;
	 use sshnb;
	  read all var {
		%do j=1 %to &maxprn;
		  llsPrin&j
		%end;
		logcs} where(_SOURCE_='SNP' & _TYPE_='SS3') into snpsscp;
	  read all var {
		%do j=1 %to &maxprn;
		  llsPrin&j
		%end;
		logcs} where(_SOURCE_='SNP*SEX' & _TYPE_='SS3') into snpXsexsscp;
	  read all var {
		%do j=1 %to &maxprn;
		  llsPrin&j
		%end;
		logcs} where(_SOURCE_='SNP*LAB' & _TYPE_='SS3') into snpXlabsscp;
	/*  read all var {
		%do j=1 %to &maxprn;
		  llsPrin&j
		%end;
		logcs} where(_SOURCE_='SEX*LAB' & _TYPE_='SS3') into sexXlabsscp; */
	  read all var {
		%do j=1 %to &maxprn;
		  llsPrin&j
		%end;
		logcs} where(_SOURCE_='LINE(SNP)' & _TYPE_='SS3') into lineinsnpsscp;
	  read all var {
		%do j=1 %to &maxprn;
		  llsPrin&j
		%end;
		logcs} where(_SOURCE_='SEX*LINE(SNP)' & _TYPE_='SS3') into sexXlineinsnpsscp;
	  read all var {
		%do j=1 %to &maxprn;
		  llsPrin&j
		%end;
		logcs} where(_SOURCE_='LAB*LINE(SNP)' & _TYPE_='SS3') into labXlineinsnpsscp;
	   read all var {
		%do j=1 %to &maxprn;
		  llsPrin&j
		%end;
		logcs} where(_TYPE_='ERROR') into errorsscp;
	close sshnb;
	* Now read the error df from ranmodnb;
	use ranmodnb;
	* With the model statement as it is snp line 2, snp*sex line 4,  snp*lab 6;
	read point {2} var {df} into df;
	close ranmodnb;

start wltest (E,H,test) global(df,testres);
	* Lambda =L= 1/(det(I+Err^-1*Hyp));
	* testres col 1 is Wilks lambda;
	testres[test,1] = 1/det(I(&numvaranl)+inv(E)*H);
	* Lambda =det(E)/det(H+E)= Prod(1/(1+l_i)=Prod(1-eigenvalues((E+H)^-1)H)  SAS manual Introduction to regression, multivariate tests.;
	* lambda formulas give the same result, and have trivial time cost.;
	* For the test, p=# of traits=maxprn, b = number of model df=1, w = residual df
	* r=w-(p-b+1)/2;

	r= df[test]-&numvaranl/2;
	* u = (pb-2)/4;
	u=(&numvaranl-2)/4;
	* t = sqrt((p^2b^2-4)/(p^2+b^2-5)) or 1 if p^2b^2-5 <0;
	* t=sqrt((&numvaranl**2-4)/(&numvaranl**2-4));
	* if t<=0 then t=1;
	t=1; * THIS IS THE SOLE DIFFERENCE BETWEEN THIS CODE AND THE ONE USED TO GENERATE THE CURRENT RESULTS (PREV 2 CODE LINES);
	* testres col 2 is the F statistic;
	testres[test,2]=(1-testres[test,1]**(1/t))/testres[test,1]**(1/t)*(r*t-2*u)/(&numvaranl);
	* df pb and rt-2u ;
	testres[test,3]=r*t-2*u;
	if testres[test,2]<=0 then testres[test,4]=1;
	else testres[test,4] =1- probF(testres[test,2],&numvaranl,r*t-2*u);
*	print r u t Fstat prob;
	* F ratio is exact when the hypothesis df <=2, which it will be for our snps.
	* Rao 1973, pg. 556.  from SAS:  F=(1-L^1/t)/L^1/t X (rt-2u)/(pb)
	* Chi-square: Test statistic W=(w-(p-b+1)/2)log(Lambda) is distributed as chisquare with bp df. Krzanowski, Wikipedia;

finish wltest;

	testres = j(1,8);
	 *Now construct synthetic error matrix for the snp effect;
	 snperrss =  &snpls*lineinsnpsscp + &snplls*labXlineinsnpsscp +&snpsls*sexXlineinsnpsscp + &snperr*errorsscp;
	 testres[1,5]=&snpls;  testres[1,6]=&snplls; testres[1,7]=&snpsls; testres[1,8]=&snperr;
	* testres[1,9]=sum(testres[1,5],testres[1,6],testres[1,7],testres[1,8]);
	 call  wltest(snperrss,snpsscp,1);

	/*
	 *Now construct synthetic error matrix for the snp*sex effect;
	 snperrss =  &snpXsexls*lineinsnpsscp + &snpXsexlls*labXlineinsnpsscp +&snpXsexsls*sexXlineinsnpsscp + &snpXsexerr*errorsscp;
	 testres[2,5]=&snpXsexls;  testres[2,6]=&snpXsexlls; testres[2,7]=&snpXsexsls; testres[2,8]=&snpXsexerr;
	* testres[2,9]=sum(testres[2,5],testres[2,6],testres[2,7],testres[2,8]);
	 call  wltest(snperrss,snpXsexsscp,2);
	 *Now construct synthetic error matrix for the snp*lab effect;
	 snperrss =  &snpXlabls*lineinsnpsscp + &snpXlablls*labXlineinsnpsscp +&snpXlabsls*sexXlineinsnpsscp + &snpXlaberr*errorsscp;
	 testres[3,5]=&snpXlabls;  testres[3,6]=&snpXlablls; testres[3,7]=&snpXlabsls; testres[3,8]=&snpXlaberr;
	* testres[3,9]=sum(testres[3,5],testres[3,6],testres[3,7],testres[3,8]);
	 call wltest(snperrss,snpXlabsscp,3);
	*/

	create snptest from testres[colname={'wilks' 'F' 'ddf' 'WltestP' 'cline(snp)' 'clabXline(snp)' 'csexXline(snp)' 'cerror'}];
	append from testres;
	quit;
run;
data resnb;
 merge testlabel snptest;
 snpid=symget('i');
run;
data resnb;
 merge resnb snpinfo;
 by snpid;
 drop snpid;
 Prin1P=symget('PvalPrin1')+0;
 Prin2P=symget('PvalPrin2')+0;
 run;
options nomlogic nomprint nosymbolgen;
data effectvecs;
 set nblsmeans; *nblsmeanbysex; 
 if dependent='logcs' then order=symget('numvaranl')+0;
 else order =substr(Dependent,8)+0;
 run;
 proc transpose data=effectvecs out=tls;
 by order;
run;
data vectornb;
 set tls;
 ave=col2-col1;
 /*F=col4-col3;
 M=col6-col5;*/
 variable= compress('Prin'||order);
 if order=&numvaranl then variable ='logcs';
run;
proc transpose data=vectornb out=lsdiffnb;
 var ave; /*F M;*/
 id variable;
run;
data together;
 length snp chunk 8.;
 merge resnb lsdiffnb;
 effectsz=sqrt(uss(of Prin1-Prin&maxprn logcs));
 effectshape=sqrt(uss(of Prin1-Prin&maxprn));
 rename _NAME_=vector;
 label _NAME_='vector';
 chunk=symget('c');
run;
/* data effnb;
  merge lsdiffnb snpinfo;
  by snpid;
  keep _NAME_ snp chrom numsnp nsnp psnp effectsz Prin1-Prin&maxprn logcs ;
  effectsz=sqrt(uss(of Prin1-Prin&maxprn logcs));
  rename _NAME_=effectsex;
  label _NAME_='effectsex';
 run; */

%if %sysfunc(exist(&libr..GWAS&maxprn&label&firstchunk.to&lastchunk)) %then %do;
 proc datasets;
  append base= &libr..GWAS&maxprn&label&firstchunk.to&lastchunk data=together;
  quit;
 run;
 %end;
 %else %do;
  data &libr..GWAS&maxprn&label&firstchunk.to&lastchunk;
   set together;
 run;
%end;

/*
%if  %sysevalf(&i/100.0)=%sysevalf(&i/100,integer) %then %do;
  proc printto log=log;
  run;
  %put Done with  &i of &nsnps %sysfunc(datetime(),datetime18.);
  run;
  proc printto log=logfile;
  run;
%end;
*/

/*%end;*/ * onemaf for nonzero maf snps;

%end;  * i for snps;


%if &output^=out %then %do;
	dm'odsresults; clear';
%end;
 options source notes;
  proc printto log=log;
  run;
  %put Done with chunk &c from &firstchunk to &lastchunk at %sysfunc(datetime(),datetime18.);
  run;
  options nosource nonotes;
  proc printto log=logfile;
  run;
 %end; * c for chunks;
proc printto;
 options source notes;
 ods listing;
 ods html;
 ods results on;
run; 
 %put END TIME: %sysfunc(datetime(),datetime14.);
 %put PROCESSING TIME:  %sysfunc(putn(%sysevalf(%sysfunc(TIME())-&datetime_start.),mmss.)) (mm:ss) ;
%mend;

*final parameter is whether you see output or not out gives more output;
* macro testSNP(firstchunk,lastchunk,maxprn,libr,label,output);
%testSNP(1,2,58,work,f2HQ,no);
* The data is split into chunks of 1000 SNPs, identified in the variable chunk in F2highq184invmsked.  You can run a range of chunks. ;
