/* This file is part of the code used to analyze data for 
Pitchers et al. 2018 (or 2019), A multivariate genome-wide association 
study of wing shape in Drosophila melanogaster to be published in Genetics. */

/* This programs read the two data files in GWAS Programs and data\MENC2:
 Maine&NC_output.csv - this is the phenotypic data.
 MNC_genotyping_processed.csv - these are the genotypes of the inbred lines

These data are then subjected to a slightly simpler version of the MANOVA analysis in MANOVA/MANOVAHighQchunk.sas.
Programs to compare the results of the DGRP and MENC2 results are not included.  
*/
 

libname alldata "D:\Dropbox\DH\Flywing\DGRP\GWAS programs and data\data";

* Genotypes from the Maine and North Carolina lines.;
PROC IMPORT OUT= WORK.MnNCgen 
            DATAFILE= "D:\Dropbox\DH\Flywing\DGRP\GWAS Programs and data\MENC2\MNC_genotyping_processed.csv" 
            DBMS=CSV REPLACE;
     GETNAMES=YES;
	 guessingrows=163;
     DATAROW=2; 
RUN;
proc transpose data=mnncgen out=tMENCgen;
 var SNP:;
 id line;
 run;
proc contents data=tMENCgen out=varnames(keep=name) noprint;
run;
proc transpose data=varnames out=tavrn;
 var NAME;
run;

* For getting a list of names of variables. ;
%macro cangen;
data longvarn;
 format longvar $1000.;
  set tavrn;
 longvar=compress(col1)
  %do q = 2 %to 163;
 || ' ' || compress(col&q)
%end;
;
 put longvar;
 run;
%mend;
%cangen;

* allfreq summarizes genotypes and frequencies by population. ;
data allfreq;
 length snp macall scall  psnpall macNC scNC psnpNC macME scME psnpME 8. stall $4.;
 format stall $4. psnpall psnpNC psnpME 6.3;
 array MElines[*] $ MA1 MA2 MA3 MA4 MA5 MA7 MA8 MA9 MA10 MA11 MA12 MA13 MA14 MA15 MA16 MA17 MA18 MA19 MA20 MA21 MA22 MA23
MA24 MA25 MA26 MA27 MA30 MA31 MA32 MA33 MA34 MA36 MA37 MA38 MA39 MA41 MA42 MA43 MA44 MA45 MA46 MA47 MA48 MA49 MA50 MA51
MA52 MA53 MA54 MA56 MA57 MA58 MA59 MA60 MA61 MA62 MA63 MA64 MA65 MA67 MA68 MA69 MA70 MA72 MA73 MA74 MA75 MA76 MA77 MA79
MA80 MA81 MA83 MA84 MA85 MA86 MA87 MA88 MA89 MA90 MA91 MA92 MA93 MA94 MA95 MA96 MA97;
array NClines[*] $ NC2 NC3 NC4 NC5 NC6 NC8 NC9 NC10 
NC11 NC12 NC13 NC14 NC15 NC16 NC17 NC18 NC19 NC20 NC21 NC22 NC23 NC24 NC26 NC27 NC28 NC29 NC30 NC31 NC32 NC35 NC37 NC38 NC39 NC40 NC41
 NC42 NC43 NC44 NC46 NC47 NC48 NC51 NC52 NC53 NC54 NC55 NC56 NC58 NC59 NC61 NC62 NC63 NC64 NC65 NC66 NC67 NC70 NC73 NC75 NC76
NC77 NC78 NC80 NC81 NC82 NC83 NC84 NC85 NC86 NC87 NC90 NC91 NC93 NC94 NC95 NC96;
array score[*] scall scNC scME;
array mac[*] macall macNC macME;
array psnp[*] psnpall psnpNC psnpME;
 set tMENCgen;
 snp = substr(_NAME_,5)+0;
 do i = 1 to 3;
	 mac[i]=0;
	 score[i]=0;
 end;
 stall='NA';
 i=0;
 do while (not(stall>'NA') & not(stall<'NA') & i<dim(of MElines));
    i=i+1;
  *  put i stall lines[i];
  if (MElines[i]>'NA' | MElines[i]<'NA') then do;
     stall =MElines[i];
	 end;
  end;
  do i=1 to dim(of MElines);
    if (MElines[i] NE 'NA') then do;
	  score[3]=score[3]+1;
	  if stall^=MElines[i] then mac[3]=mac[3]+1;
	end;
  end;
  do i=1 to dim(of NClines);
    if (NClines[i] NE 'NA') then do;
	  score[2]=score[2]+1;
	  if stall^=NClines[i] then mac[2]=mac[2]+1;
	end;
  end; 
  mac[1]=mac[2]+mac[3];
  score[1]=score[2]+score[3];
  psnp[1]=mac[1]/score[1];
  if psnp[1]>0.5 then do;
    do i =1 to 3;
	   mac[i]=score[i]-mac[i];
	end;
  end;
  psnp[1]=mac[1]/score[1];
  psnp[2]=mac[2]/score[2];
  psnp[3]=mac[3]/score[3];
 drop i;
run;

data macg3;
 length snp case macall scall  psnpall macNC scNC psnpNC macME scME psnpME 8. stall $4.;
 set allfreq;
 where macall>2;
 if macNC>0 & macME>0 then case=1;
 else if macNC>0 then case=2;
 else case=3;
run;
proc sgplot;
 scatter x=psnpNC y=psnpME;
run;
  
proc transpose data=macg3 out=var2anal;
 var MA1 MA2 MA3 MA4 MA5 MA7 MA8 MA9 MA10 MA11 MA12 MA13 MA14 MA15 MA16 MA17 MA18 MA19 MA20 MA21 MA22 MA23
MA24 MA25 MA26 MA27 MA30 MA31 MA32 MA33 MA34 MA36 MA37 MA38 MA39 MA41 MA42 MA43 MA44 MA45 MA46 MA47 MA48 MA49 MA50 MA51
MA52 MA53 MA54 MA56 MA57 MA58 MA59 MA60 MA61 MA62 MA63 MA64 MA65 MA67 MA68 MA69 MA70 MA72 MA73 MA74 MA75 MA76 MA77 MA79
MA80 MA81 MA83 MA84 MA85 MA86 MA87 MA88 MA89 MA90 MA91 MA92 MA93 MA94 MA95 MA96 MA97 NC2 NC3 NC4 NC5 NC6 NC8 NC9 NC10 
NC11 NC12 NC13 NC14 NC15 NC16 NC17 NC18 NC19 NC20 NC21 NC22 NC23 NC24 NC26 NC27 NC28 NC29 NC30 NC31 NC32 NC35 NC37 NC38 NC39 NC40 NC41
 NC42 NC43 NC44 NC46 NC47 NC48 NC51 NC52 NC53 NC54 NC55 NC56 NC58 NC59 NC61 NC62 NC63 NC64 NC65 NC66 NC67 NC70 NC73 NC75 NC76
NC77 NC78 NC80 NC81 NC82 NC83 NC84 NC85 NC86 NC87 NC90 NC91 NC93 NC94 NC95 NC96;
id _NAME_;
run;
proc sort;
 by _NAME_;
 run;

 * This is the phenotype data on the Maine and NC lines.;
 * Flies were all male, so no sex to worry about.;
PROC IMPORT OUT= WORK.MNWing 
            DATAFILE= "D:\Dropbox\DH\Flywing\DGRP\GWAS Programs and data\MENC2\Maine&NC_output.csv" 
            DBMS=CSV REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;

data MNwingup;
  format line $8.;
  array xy[*] x1-x48 y1-y48;
  set MNwing;
  do i = 1 to dim(of xy);
    xy[i]=xy[i]*100;
  end;
  logcs=log10(csize);
  drop i;
run;
proc score data=MNwingup score=alldata.Interlableigjun out=scMNwing;
run;
proc sort;
 by line rep;
 run;
proc means noprint;
 by line rep;
 var Prin1-Prin58 logcs;
 output out=llsm mean=llsPrin1-llsPrin58 logcs;
 run;

proc means mean var;
 var llsprin1-llsprin58 logcs;
run;


%macro analyzeMN(ranreps,libr,sizeh,label,maxprn,out);
options nomlogic nomprint nosymbolgen;
%if &sizeh = szleft %then %let numvaranl=%eval(&maxprn+1);
%else %let numvaranl=&maxprn;
%put numvaranl=&numvaranl;
*options mlogic mprint symbolgen;
data _NULL_;
	if 0 then set macg3 nobs=n;
	call symputx('ndat',n);
	stop;
run;
%put nobs=&ndat;
 %if &out^=yes %then %do;
  ods listing close; * Turns off Output to Output window.;  
  ods html close;  * These two commands turn off the Results Tracking and Results viewer output.;
  ods results off;
  options nosource nonotes;
 run;
 %end;
%*do s = 1 %to &ndat; 
%do s = 1 %to 1;  * case 1;

%*do s = 22 %to 22;  * case 2;
%*do s = 99 %to 99;  * case 3;
  data which ;
   set macg3 (firstobs=&s obs=&s);
	call symputx('SNP',_NAME_);
	call symputx('case',case);
  run;
	data testlabel;
	 effect = symget('snp');
	 output;
	 run;
  data snpinfo;
    set which;
	keep snp case macall scall psnpall macNC scNC psnpNC macME scME psnpME stall _NAME_;
	label _NAME_='csnp';
	rename _NAME_ =csnp;
  run;
  %*put SNP=&SNP;
   data wsnp;
    set var2anal;
	where &SNP^='NA';
	label _NAME_=line;
	 pop=substr(_NAME_,1,2);
	 keep _NAME_ pop &SNP;
	 rename _NAME_=line;
   run;
   %do r=0 %to &ranreps;
   	%if &r>0 %then %do;
		data chord;
		 iseed=0;
		 set wsnp;
		 ord=ranuni(iseed);
		 keep &snp pop ord;
		run;
		proc sort data=chord;
		 by pop ord;
		 run;
		data wsnp;
		 merge wsnp(keep=line) chord;
		run;
	%end;
		   data snpwing;
		    length line pop lnnum $8. &SNP $15.;
		     merge wsnp(in=A) llsm;
			 by line;
			 if A;
			 lnnum=substr(line,3);
			 keep line lnnum pop &snp rep _FREQ_ llsPrin1-llsPrin58 logcs;
			 run ;
		%if &case=1 %then %do;
		  %*put case=&case;
				 * GLM1. This univariate run of GLM is to get the weights used to construct the denominator MS.;
			ods output RandomModelANOVA=ranmod;
			proc glm data=snpwing; 
			 class &snp pop line;
			 *if you change the model and random statements, correct the iml code for reading df!;
			%if &sizeh=szright %then %do; model llsPrin1= pop &snp &snp*pop line(&snp pop) logcs;
			%end; %else %do; model llsPrin1= pop &snp &snp*pop line(&snp pop);  %end;
			  random line(&snp pop) / test;
			  weight _FREQ_;
			 run;
			ods output close;
			quit;
			run;
			
		  	 %let sourceshsnp=LINE(%substr(&snp,1,10)*POP);
		 %end;
		 %else %if &case=2 %then %do;
		   %*put case 2;
		   * Case is only data for NC;
		   	ods output RandomModelANOVA=ranmod;
			proc glm data=snpwing(where=(pop='NC')); 
			 class &snp line;
			 *if you change the model and random statements, correct the iml code for reading df!;
			 %if &sizeh=szright %then %do; model llsPrin1= &snp line(&snp) logcs;
			 %end; %else %do; model llsPrin1= &snp line(&snp);  %end;
			  random line(&snp) / test;
			  weight _FREQ_;
			 run;
			ods output close;
			quit;
			run;

		  	 %let sourceshsnp=LINE(&snp);
		 %end;
		 %else %do;
		   * Case 3 is only data for ME;
		   %*put case 3;
		    ods output RandomModelANOVA=ranmod;
			proc glm data=snpwing(where=(pop='MA')); 
			 class &snp line;
			 *if you change the model and random statements, correct the iml code for reading df!;
			 %if &sizeh=szright %then %do; model llsPrin1= &snp line(&snp) logcs;
			 %end; %else %do; model llsPrin1= &snp line(&snp);  %end;
			  random line(&snp) / test;
			  weight _FREQ_;
			 run;
			ods output close;
			quit;
			run;
		  	 %let sourceshsnp=LINE(&snp);
			%end;

			* The next section of code is for reading back the weights for the SSCP matrices;

				%let snplsp=0; 
				%let snperr=0;
				option spool;

				**** ;
				* This data step parses the definition of the denominator MS into the weights for each type.
				* It creates macro variables for each weight to carry forward to the IML step.;
				* It also saves the weights in the data set for future reference.;
				data ranmod;
				 set ranmod;
				 source=upcase(source);
				 error=upcase(error);
				run;
				data coeffsnb;
				 attrib error length=$120; 
				  format error $120.;
				 set ranmod;
				  where Source="&snp";
				 done=0;
				 sign=1;
				 remains=error;
				 nextstar = index(remains,'*'); 
				 *iter=0;
				 do until (done=1);
				    wcoeff= (compress(scan(remains,1,'*'))+0)*sign;
					remains = substr(remains,nextstar+1);
					nword =compress(scan(remains,1,'+-'));
					*iter=iter+1;
					*put iter 'wcoeff ' wcoeff 'remains ' remains 'nword ' nword 'sign ' sign;
					if index(nword,"LINE(")>0 then call symput('snplsp',wcoeff);
					 else if index(nword,'ERROR')>0 then call symput('snperr',wcoeff);
					nextstar = index(remains,'*'); 
					if nextstar=0 then done=1;
					else do;
					    nextsign = indexc(remains,'+-'); 
						*put iter 'remains ' remains  'nextsign ' nextsign;
						signn = substr(remains,nextsign,1);
						*put iter 'remains ' remains  'nextsign ' nextsign 'signn ' signn;
						remains=substr(remains,nextsign+1);
						nextstar = index(remains,'*'); 
						if signn='-' then sign=-1;
						else sign=1;
						*put iter 'remains ' remains 'nextstar ' nextstar 'nextsign ' nextsign 'signn ' signn;
					  end; * done;
				 end;  * loop for parsing;
				 run;
				 %*put &snplsp &snperr &case;
		  * GLM 2. This is the run of GLM that actually calculates the SSCP matrices. The SSCP matrices are saved in the data set sshnb.;
			 %if &case=1 %then %do;
			   %*put case 1 for GLM 2;
			ods output lsmeans=nblsmeans;
		     proc glm data=snpwing outstat=sshnb; 
			 class &snp pop line;
			 *if you change the model and random statements, correct the iml code for reading df!;
				  %if &sizeh=szright %then %do; model llsPrin1-llsPrin&maxprn=pop &snp line(&snp pop) logcs / nouni;
				  %end; %else %do; model llsPrin1-llsPrin&maxprn logcs=pop &snp line(&snp pop) / nouni; %end;
				  random line(&snp pop) / test;
			     weight _FREQ_;
			 	 lsmeans &snp;
				 run;
			%end;
			 %else %if &case=2 %then %do;
			   %*put case 2 for GLM 2;
			   * Case is only data for NC;
			ods output lsmeans=nblsmeans;
				proc glm data=snpwing(where=(pop='NC')) outstat=sshnb; 
				 class &snp line;
				 *if you change the model and random statements, correct the iml code for reading df!;
				 %if &sizeh=szright %then %do; model llsPrin1-llsPrin&maxprn= &snp line(&snp) logcs/ nouni;
				 %end; %else %do; model llsPrin1-llsPrin&maxprn logcs= &snp line(&snp) / nouni; %end;
				  random line(&snp) / test;
			     weight _FREQ_;
			 	 lsmeans &snp;
				 run;
		  	 %let sourceshsnp=LINE(&snp);
			%end;
			 %else %if &case=3 %then %do;
			   %*put case 3 for GLM 2;
			   * Case is only data for NC;
			ods output lsmeans=nblsmeans;
				proc glm data=snpwing(where=(pop='MA')) outstat=sshnb; 
				 class &snp line;
				 *if you change the model and random statements, correct the iml code for reading df!;
				 %if &sizeh=szright %then %do; model llsPrin1-llsPrin&maxprn= &snp line(&snp) logcs/ nouni;
				 %end; %else %do; model llsPrin1-llsPrin&maxprn logcs= &snp line(&snp) / nouni; %end;
				  random line(&snp) / test;
			     weight _FREQ_;
			 	 lsmeans &snp;
				 run;
				%end;
			ods output close;
			quit;
		    run;
			data sshnb;
			 set sshnb;
			  _SOURCE_=upcase(_SOURCE_);
			run;

		 * IML is now run to weight the SSCP matrices by the coefficients estimated in GLM run 1,
		then calculate Wilks lambda and other statistics for the MANOVA.;
		  %*put &sourceshsnp;
		 proc iml;
			 use sshnb;
			  read all var {
				%do j=1 %to &maxprn;
				  llsPrin&j
				%end;
				%if &sizeh=szleft %then %do; logcs %end;
     			} where(_SOURCE_="&snp" & _TYPE_='SS3') into snpsscp;
			  read all var {
				%do j=1 %to &maxprn;
				  llsPrin&j
				%end;
				%if &sizeh=szleft %then %do; logcs %end;
				} where(_SOURCE_="&sourceshsnp"  & _TYPE_='SS3') into lineinsnppopsscp;
			%if &case=1 %then %do;
			  read all var {
				%do j=1 %to &maxprn;
				  llsPrin&j
				%end;
				%if &sizeh=szleft %then %do; logcs %end;
				} where(_SOURCE_="POP" & _TYPE_='SS3') into popsscp;
		  	%end;
				read all var {
				%do j=1 %to &maxprn;
				  llsPrin&j
				%end;
				%if &sizeh=szleft %then %do; logcs %end;
				} where(_TYPE_='ERROR') into errorsscp;
			close sshnb;
			*print (ncol(errorsscp)) (nrow(errorsscp)) (ncol(lineinsnppopsscp)) (nrow(lineinsnppopsscp));
			* Now read the error df from ranmod;
			use ranmod;
			* With the model statement as it is snp line 2, snp*sex line 4,  snp*lab 6;
			read point {4} var {df} into df;
			close ranmod;

		start wltest (E,H,test) global(df,testres);
			* Lambda =L= 1/(det(I+Err^-1*Hyp));
			* testres col 1 is Wilks lambda;
			testres[test,1] = 1/det(I(&numvaranl)+inv(E)*H);
			* Lambda =det(E)/det(H+E)= Prod(1/(1+l_i)=Prod(1-eigenvalues((E+H)^-1)H)  SAS manual Introduction to regression, multivariate tests.;
			* lambda formulas give the same result, and have trivial time cost.;
			* For the test, p=# of traits=maxprn, b = number of model df=1, w = residual df
			* r=w-(p-b+1)/2;

			r= df[test]-&numvaranl/2;
			* u = (pb-2)/4;
			u=(&numvaranl-2)/4;
			* t = sqrt((p^2b^2-4)/(p^2+b^2-5)) or 1 if p^2b^2-5 <0;
			* t=sqrt((&numvaranl**2-4)/(&numvaranl**2-4));
			* if t<=0 then t=1;
			t=1; * THIS IS THE SOLE DIFFERENCE BETWEEN THIS CODE AND THE ONE USED TO GENERATE THE CURRENT RESULTS (PREV 2 CODE LINES);
			* testres col 2 is the F statistic;
			testres[test,2]=(1-testres[test,1]**(1/t))/testres[test,1]**(1/t)*(r*t-2*u)/(&numvaranl);
			* df pb and rt-2u ;
			testres[test,3]=r*t-2*u;
			if testres[test,2]<=0 then testres[test,4]=1;
			else testres[test,4] =1- probF(testres[test,2],&numvaranl,r*t-2*u);
		*	print r u t Fstat prob;
			* F ratio is exact when the hypothesis df <=2, which it will be for our snps.
			* Rao 1973, pg. 556.  from SAS:  F=(1-L^1/t)/L^1/t X (rt-2u)/(pb)
			* Chi-square: Test statistic W=(w-(p-b+1)/2)log(Lambda) is distributed as chisquare with bp df. Krzanowski, Wikipedia;
		finish wltest;

			testres = j(1,6);
			 *Now construct synthetic error matrix for the snp effect;
			 snperrss =  &snplsp*lineinsnppopsscp  + &snperr*errorsscp;
			 testres[1,5]=&snplsp;  testres[1,6]=&snperr;
			 call  wltest(snperrss,snpsscp,1);
			create snptest from testres[colname={'wilks' 'F' 'ddf' 'WltestP' 'cline(snp*pop)' 'cerror'}];
			append from testres;
		/* 	%if &case=1 %then %do;
			 call  wltest(snperrss,popsscp,1);
			print 'For pop ' testres;
		  	%end; */
			quit; 
		run;
		 data resnb;
		 merge testlabel snptest;
		run;
		data resnb;
		 merge resnb snpinfo;
		 run;
		data effectvecs;
		 set nblsmeans; 
		 if index(Dependent,'lls')>0 then order =substr(Dependent,8)+0;
		 else order=59;
		 run;
		 proc transpose data=effectvecs out=tls;
		 by order;
		run;
		data vectornb;
		 set tls;
		 ave=col2-col1;

		 variable= compress('Prin'||order);
		run;
		proc transpose data=vectornb out=lsdiffnb;
		 var ave;
		 id variable;
		run;

		data together;
		 length snp 8. rrep $2.;
		 merge resnb lsdiffnb;
		 effectshape=sqrt(uss(of Prin1-Prin58));
	     rrep = compress(symget('r'));
		run;


		%if %sysfunc(exist(&libr..MENC&label&ranreps&numvaranl)) %then %do;
		 proc datasets;
		  append base= &libr..MENC&label&ranreps&numvaranl data=together;
		  quit;
		 run;
		 %end;
		 %else %do;
		  data &libr..MENC&label&ranreps&numvaranl;
		   set together;
		 run;
		%end;
	%end; *r rand reps.;
 %put Finished SNP &s of &ndat;
	 %if &out^=yes %then %do;
	  dm'odsresults; clear';
	 options nosource nonotes;
	%end; 
%end;  * s;
 ods listing;
  ods html;
  ods results on;
  options source notes;
  proc printto;
  run;
%mend;
* This analyzes the ME and NC data, with population as an effect, and then resamples the data 
ranreps times, and get results for randomized data.;
* macro analyzeMN(ranreps,libr,sizeh,label,maxprn,out);
%analyzeMN(20,work,szright,lcszr,58,no);

